class Vertex:
    def __init__(self, location=(0, 0, 0)):
        self.location = location
        self.neighbors = [None, None, None]


class Tile:
    def __init__(self, resource='Wheat', location=(0, 0, 0)):
        self.__resource = resource
        self.__robber = False
        self.__location = location
        self.__roads = []
        self.__buildings = []
        self.__number = 0
        # roads and building list start due north and iterate clockwise

    def get_number(self):
        return self.__number

    def set_number(self, integer):
        self.__number = integer

    def get_resource(self):
        return self.__resource

    def set_robber(self, value):
        self.__robber = value

    # def update_buildings(self, index, player):
    #    self.__buildings[index]

    def get_robber(self):
        return self.__robber

    def get_buildings(self):
        return self.__buildings

    def get_location(self):
        return self.__location

    def set_location(self, location):
        self.__location = location

    robber = property(get_robber, set_robber)
    buildings = property(get_buildings)  # , update_buildings)
    resource = property(get_resource)
    location = property(get_location, set_location)
    number = property(get_number, set_number)


class Port:
    def __init__(self, location=(0, 0, 0)):
        self.location = location
        self.specialty = False
        self.trade = 'None'
        self.is_port = False
        self.rate = 4

    def set_property(self, location, resource='None'):
        self.location = location
        self.trade = resource

    def get_type(self):
        return self.trade

    def set_type(self, trade):
        if trade == 'None':
            self.rate = 3
        else:
            self.rate = 2
        self.trade = trade
        self.is_port = True

    def get_location(self):
        return self.location

    def set_location(self, location):
        self.location = location

    resource = property(get_type, set_type)


class Board:
    def __init__(self):
        self.num_resource_tiles = 19
        self.num_ocean_tiles = 18
        self.num_ports = 9
        self.ocean_location_list = []
        self.fill_ocean_location_list()
        self.lookup_table = {2: [], 3: [], 4: [], 5: [], 6: [], 7: [], 8: [], 9: [], 10: [], 11: [], 12: []}
        self.location_list = []
        self.fill_location_list()
        self.__ocean_tiles = []
        self.ports = []
        self.robber = None
        self.__resource_tiles = []
        self.direction_array = [
            (0, 1, -1),  # top left
            (1, 0, -1),  # top right
            (1, -1, 0),  # right
            (0, -1, 1),  # bottom right
            (-1, 0, 1),  # bottom left
            (-1, 1, 0)  # left
        ]

        # 2d array to reference tiles by coordinate
        self.__array = []
        for index in range(0, 7):
            self.__array.append([0, 0, 0, 0, 0, 0, 0])
        self.translate_to_3d()

    def form_ocean(self):
        pass

    def get_array(self):
        return self.__array

    def translate_to_3d(self):
        for tile in self.__resource_tiles:
            self.__array[int(tile.location[0]) + 3][int(tile.location[1]) + 3] = tile
        for port in self.ports:
            self.__array[int(port.location[0]) + 3][int(port.location[1]) + 3] = port

    def get_tile(self, tup):
        return self.__array[int(tup[0] + 3)][int(tup[1] + 3)]

    def get_resource_list(self):
        return self.__resource_tiles

    def get_ocean_list(self):
        return self.__ocean_tiles

    def get_port_list(self):
        return self.ports

    def generate_land(self, number_list, resource_list):
        count = 0

        # assign tile resource
        for index in self.location_list:
            self.__resource_tiles.append(
                Tile(resource_list[count], index))
            count += 1

        # assign tile numbers
        index = 0
        for tile in self.__resource_tiles:
            if tile.resource == 'Desert':
                tile.number = 7
                tile.set_robber(True)
                self.robber = tile
            else:
                tile.number = number_list[index]
                self.lookup_table[number_list[index]].append(tile)
                index += 1

    def xy_give_port(self, location):
        valid_ports = \
            [
                [0, 0, 1, 1, 0, 2, 2, 0, 0, 0, 0],
                [0, 9, 0, 0, 0, 0, 0, 0, 3, 3, 0],
                [0, 9, 0, 0, 0, 0, 0, 0, 0, 0, 4],
                [0, 8, 0, 0, 0, 0, 0, 0, 0, 0, 4],
                [0, 8, 0, 0, 0, 0, 0, 0, 5, 5, 0],
                [0, 0, 7, 7, 0, 6, 6, 0, 0, 0, 0]]
        index = valid_ports[location[1]][location[0]]
        if index == 0:
            return None
        else:
            return self.ports[index - 1]

    def generate_ocean(self):
        # add ocean, not really used
        for index in self.ocean_location_list:
            self.__ocean_tiles.append(Port(index))

    def print_ports(self):
        for port in self.__ocean_tiles:
            if port.is_port:
                print(port.location, port.rate, port.resource)

    def generate_ports(self, port_list):
        self.generate_ocean()
        index = 0
        even = 0
        for tile in self.__ocean_tiles:
            if even % 2 == 0:
                tile.set_type(port_list[index])
                self.ports.append(tile)
                index += 1
            even += 1

    def set_robber(self, location):
        self.robber.set_robber(False)
        self.robber = self.get_tile(location)
        self.robber.set_robber(True)

    def get_robber(self):
        return (self.robber)

    def fill_ocean_location_list(self):
        self.ocean_location_list.append([0, 3, -3])
        self.ocean_location_list.append([1, 2, -3])
        self.ocean_location_list.append([2, 1, -3])
        self.ocean_location_list.append([3, 0, -3])
        self.ocean_location_list.append([3, -1, -2])
        self.ocean_location_list.append([3, -2, -1])
        self.ocean_location_list.append([3, -3, 0])
        self.ocean_location_list.append([2, -3, 1])
        self.ocean_location_list.append([1, -3, 2])
        self.ocean_location_list.append([0, -3, 3])
        self.ocean_location_list.append([-1, -2, 3])
        self.ocean_location_list.append([-2, -1, 3])
        self.ocean_location_list.append([-3, 0, 3])
        self.ocean_location_list.append([-3, 1, 2])
        self.ocean_location_list.append([-3, 2, 1])
        self.ocean_location_list.append([-3, 3, 0])
        self.ocean_location_list.append([-2, 3, -1])
        self.ocean_location_list.append([-1, 3, -2])

    def fill_location_list(self):
        self.location_list.append([0, 2, -2])
        self.location_list.append([1, 1, -2])
        self.location_list.append([2, 0, -2])
        self.location_list.append([-1, 2, -1])
        self.location_list.append([0, 1, -1])
        self.location_list.append([1, 0, -1])
        self.location_list.append([2, -1, -1])
        self.location_list.append([-2, 2, 0])
        self.location_list.append([-1, 1, 0])
        self.location_list.append([0, 0, 0])
        self.location_list.append([1, -1, 0])
        self.location_list.append([2, -2, 0])
        self.location_list.append([-2, 1, 1])
        self.location_list.append([-1, 0, 1])
        self.location_list.append([0, -1, 1])
        self.location_list.append([1, -2, 1])
        self.location_list.append([-2, 0, 2])
        self.location_list.append([-1, -1, 2])
        self.location_list.append([0, -2, 2])

    land_list = property(get_resource_list)
    ocean_list = property(get_ocean_list)
    port_list = property(get_port_list)



class Tile:
    def __init__(self, resource='Ocean', location=[0, 0, 0]):
        self.__resource = resource
        self.__robber = False
        self.__location = location

    def get_resource(self):
        return self.__resource

    def set_robber(self, value):
        self.__robber = value

    def get_robber(self):
        return self.__robber

    def on_roll(self):
        if self.__robber:
            pass
        else:
            pass
        # iterate settlements-cities around this tile
        # each get 1-2 of this ones resource

    def get_location(self):
        return self.__location

    def set_location(self, location):
        self.__location = location

    robber = property(get_robber, set_robber)
    resource = property(get_resource)
    location = property(get_location, set_location)



import math


class Hexagon:
    def __init__(self, radius, flat=True):
        self.__radius = radius
        self.__points = []
        self.__flat = flat
        self.construct_hexagon()

    def construct_hexagon(self):
        # clockwise starting from north
        width = 0.5 * self.__radius
        height = self.__radius * math.sin(math.pi / 3)
        if self.__flat:
            # construct generic hexagon
            self.__points = [(0 + width, 0 + height),
                             (self.__radius, 0),
                             (0 + width, 0 - height),
                             (0 - width, 0 - height),
                             (-self.__radius, 0),
                             (0 - width, 0 + height)]
        else:
            self.__points = [(0, self.__radius),
                             (height, width),
                             (height, -width),
                             (0, -self.__radius),
                             (-height, -width),
                             (-height, width)]

    def get_points(self):
        return self.__points

    def offset_flat(self, location, origin):
        if location == (0, 0, 0):
            return self.__points

        y_constant = self.__radius * (3 ** 0.5) / 2

        if location[0] == 0:
            x_off = 0
        elif location[0] % 2 == 0:
            if location[0] > 0:
                x_off = self.__radius * (location[0] + 1)
            else:
                x_off = self.__radius * (location[0] - 1)
        else:
            if location[0] > 0:
                x_off = (location[0] + 0.5) * self.__radius
            else:
                x_off = (location[0] - 0.5) * self.__radius

        y_off = (location[1] - location[2]) * y_constant

        result = []

        for point in self.__points:
            result.append((int(point[0] + x_off + origin[0]),
                           int(point[1] - y_off + origin[1])))
        return result

    def offset_pointy(self, location, origin):
        if location == (0, 0, 0):
            return self.__points

        y_constant = self.__radius * (3 ** 0.5) / 2

        if location[0] == 0:
            x_off = 0
        elif location[0] % 2 == 0:
            if location[0] > 0:
                x_off = self.__radius * (location[0] + 1)
            else:
                x_off = self.__radius * (location[0] - 1)
        else:
            if location[0] > 0:
                x_off = (location[0] + 0.5) * self.__radius
            else:
                x_off = (location[0] - 0.5) * self.__radius

        y_off = (location[1] - location[2]) * y_constant

        result = []

        for point in self.__points:
            result.append((int(point[0] + y_off + origin[0]),
                           int(point[1] - x_off + origin[1])))
        return result

    points = property(get_points)



import pygame
import pygame_gui as pg_g
from frontiersman.client.GuiConstants import CARD_SIZE, SPACING

CARDS = ['Brick', 'Ore', 'Sheep', 'Wheat', 'Wood']


class TradeGui:
    def __init__(self, top_left, resource_hand, trade_ratios, manager, callback, container=None):
        self.resource_hand = resource_hand
        self.trade_ratios = trade_ratios
        self.manager = manager
        self.callback = callback
        self.is_valid = False
        self.error_text = ""

        self.panel = pg_g.elements.UIPanel(
            relative_rect=pygame.Rect(top_left, ((CARD_SIZE[0] + SPACING) * 20, (CARD_SIZE[1] + SPACING) * 3)),
            starting_layer_height=1,
            manager=self.manager,
            container=container,
        )
        self.give_panel = pg_g.elements.UIPanel(
            relative_rect=pygame.Rect(0, 0, self.panel.relative_rect.width / 2, CARD_SIZE[1] + 2 * SPACING),
            starting_layer_height=1,
            manager=self.manager,
            container=self.panel,
        )
        self.give_card_keys = []
        self.give_card_elements = []
        self.take_panel = pg_g.elements.UIPanel(
            relative_rect=pygame.Rect(self.give_panel.relative_rect.right, 0, self.panel.relative_rect.width / 2, CARD_SIZE[1] + 2 * SPACING),
            starting_layer_height=1,
            manager=self.manager,
            container=self.panel,
        )
        self.take_card_keys = []
        self.take_card_elements = []

        card_offset = (self.panel.relative_rect.width / 2 - 5 * CARD_SIZE[0]) / 6

        self.give_buttons = [
            pg_g.elements.UIButton(
                relative_rect=pygame.Rect(
                    (self.give_panel.relative_rect.left + card_offset + i * (CARD_SIZE[0] + card_offset), self.give_panel.relative_rect.bottom + SPACING), CARD_SIZE),
                text="",
                manager=self.manager,
                container=self.panel,
                object_id="#" + card,
            ) for i, card in enumerate(CARDS)
        ]
        self.take_buttons = [
            pg_g.elements.UIButton(
                relative_rect=pygame.Rect(
                    (self.take_panel.relative_rect.left + card_offset + i * (CARD_SIZE[0] + card_offset), self.take_panel.relative_rect.bottom + SPACING), CARD_SIZE),
                text="",
                manager=self.manager,
                container=self.panel,
                object_id="#" + card
            ) for i, card in enumerate(CARDS)
        ]

        pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(
                self.give_buttons[0].relative_rect.left, self.give_buttons[0].relative_rect.bottom + SPACING,
                self.give_buttons[0].relative_rect.width, 35
            ),
            html_text="4:1",  # todo replace with value from self.trade_ratios
            manager=self.manager,
            container=self.panel,
        )

        pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(
                self.give_buttons[1].relative_rect.left, self.give_buttons[1].relative_rect.bottom + SPACING,
                self.give_buttons[1].relative_rect.width, 35
            ),
            html_text="4:1",  # todo replace with value from self.trade_ratios
            manager=self.manager,
            container=self.panel,
        )

        pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(
                self.give_buttons[2].relative_rect.left, self.give_buttons[2].relative_rect.bottom + SPACING,
                self.give_buttons[1].relative_rect.width, 35
            ),
            html_text="4:1",  # todo replace with value from self.trade_ratios
            manager=self.manager,
            container=self.panel,
        )

        pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(
                self.give_buttons[3].relative_rect.left, self.give_buttons[3].relative_rect.bottom + SPACING,
                self.give_buttons[3].relative_rect.width, 35
            ),
            html_text="4:1",  # todo replace with value from self.trade_ratios
            manager=self.manager,
            container=self.panel,
        )

        pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(
                self.give_buttons[4].relative_rect.left, self.give_buttons[4].relative_rect.bottom + SPACING,
                self.give_buttons[4].relative_rect.width, 35
            ),
            html_text="4:1",  # todo replace with value from self.trade_ratios
            manager=self.manager,
            container=self.panel,
        )

        button_width = 300
        button_spacing = (self.take_panel.relative_rect.width - 2 * button_width) / 3
        self.trade_with_bank_button = pg_g.elements.UIButton(
            relative_rect=pygame.Rect(
                self.take_panel.relative_rect.left + button_spacing, self.take_buttons[0].relative_rect.bottom + SPACING,
                button_width, 45
            ),
            text="Trade with bank",
            manager=self.manager,
            container=self.panel,
        )
        self.trade_with_player_button = pg_g.elements.UIButton(
            relative_rect=pygame.Rect(
                self.take_panel.relative_rect.left + 2 * button_spacing + button_width, self.take_buttons[0].relative_rect.bottom + SPACING,
                button_width, 45
            ),
            text="Trade with players",
            manager=self.manager,
            container=self.panel,
        )
        # todo remove line
        self.trade_with_player_button.disable()
        self.info_text = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(
                0, self.trade_with_bank_button.relative_rect.bottom, self.panel.relative_rect.width - 2 * SPACING, 35
            ),
            html_text=self.error_text,
            manager=self.manager,
            container=self.panel
        )

        self.validate_trade()

    def kill(self):
        self.panel.kill()

    def update_give_cards(self):
        for elem in self.give_card_elements:
            elem.kill()
        self.give_card_elements = []

        if len(self.give_card_keys) > 1:
            card_width = (self.give_panel.relative_rect.width - CARD_SIZE[0]) / (len(self.give_card_keys) - 1)
        else:
            card_width = CARD_SIZE[0] + SPACING
        if card_width >= CARD_SIZE[0] + SPACING:
            card_width = CARD_SIZE[0] + SPACING
            offset = (self.give_panel.relative_rect.width - card_width * len(self.give_card_keys)) / 2
        else:
            offset = 0

        for i, card in enumerate(self.give_card_keys):
            self.give_card_elements.append(pg_g.elements.UIButton(
                relative_rect=pygame.Rect((offset + i * card_width, SPACING), CARD_SIZE),
                text="",
                manager=self.manager,
                container=self.give_panel,
                object_id="#" + card
            ))

        self.validate_trade()

    def update_take_cards(self):
        for elem in self.take_card_elements:
            elem.kill()
        self.take_card_elements = []

        if len(self.take_card_keys) > 1:
            card_width = (self.give_panel.relative_rect.width - CARD_SIZE[0]) / (len(self.take_card_keys) - 1)
        else:
            card_width = CARD_SIZE[0] + SPACING
        if card_width >= CARD_SIZE[0] + SPACING:
            card_width = CARD_SIZE[0] + SPACING
            offset = (self.give_panel.relative_rect.width - card_width * len(self.take_card_keys)) / 2
        else:
            offset = 0

        for i, card in enumerate(self.take_card_keys):
            self.take_card_elements.append(pg_g.elements.UIButton(
                relative_rect=pygame.Rect((offset + i * card_width, SPACING), CARD_SIZE),
                text="",
                manager=self.manager,
                container=self.take_panel,
                object_id="#" + card
            ))

        self.validate_trade()

    def validate_trade(self):
        counts = [0, 0, 0, 0, 0]
        for card in self.give_card_keys:
            counts[CARDS.index(card)] += 1
        brick, ore, wool, grain, lumber = counts

        # check if player has enough resources
        if brick >= self.resource_hand.brick:
            self.give_buttons[0].disable()
        else:
            self.give_buttons[0].enable()

        if ore >= self.resource_hand.ore:
            self.give_buttons[1].disable()
        else:
            self.give_buttons[1].enable()

        if wool >= self.resource_hand.wool:
            self.give_buttons[2].disable()
        else:
            self.give_buttons[2].enable()

        if grain >= self.resource_hand.grain:
            self.give_buttons[3].disable()
        else:
            self.give_buttons[3].enable()

        if lumber >= self.resource_hand.lumber:
            self.give_buttons[4].disable()
        else:
            self.give_buttons[4].enable()

        # prevent too many cards in request hand, todo check this
        if brick + ore + wool + grain + lumber > 7:
            for button in self.take_buttons:
                button.disable()
        else:
            for button in self.take_buttons:
                button.enable()

        # check if legal trade with bank
        points = 0
        while brick >= self.trade_ratios.brick:
            brick -= self.trade_ratios.brick
            points += 1
        while ore >= self.trade_ratios.ore:
            ore -= self.trade_ratios.ore
            points += 1
        while wool >= self.trade_ratios.wool:
            wool -= self.trade_ratios.wool
            points += 1
        while grain >= self.trade_ratios.grain:
            grain -= self.trade_ratios.grain
            points += 1
        while lumber >= self.trade_ratios.lumber:
            lumber -= self.trade_ratios.lumber
            points += 1

        delta_points = points - len(self.take_card_keys)
        counts = [brick, ore, wool, grain, lumber]
        self.is_valid = True

        for i, amount in enumerate(counts):
            if amount > 0:
                self.is_valid = False
                if delta_points < 0:
                    self.error_text = "The bank demands more " + CARDS[i].lower() + "."
                else:
                    self.error_text = "The bank refuses too much " + CARDS[i].lower() + "."
                break

        if self.is_valid:
            if delta_points > 0:
                self.is_valid = False
                self.error_text = "The bank refuses your overly generous offer."
            elif delta_points < 0:
                self.is_valid = False
                self.error_text = "The bank demands more for these resources."
            elif points == 0:
                self.is_valid = False
                self.error_text = "Offer a trade to the bank or to other players."
            else:
                self.error_text = "The bank likes your offer."

        self.info_text.kill()
        self.info_text = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(
                0, self.trade_with_bank_button.relative_rect.bottom, self.panel.relative_rect.width - 2 * SPACING, 35
            ),
            html_text=self.error_text,
            manager=self.manager,
            container=self.panel
        )
        if self.is_valid:
            self.trade_with_bank_button.enable()
        else:
            self.trade_with_bank_button.disable()

    def handle_ui_button_pressed(self, event):  # Todo
        if event.ui_element in self.give_buttons:
            try:
                index = self.give_buttons.index(event.ui_element)
                self.give_card_keys.append(CARDS[index])
                self.update_give_cards()
            except ValueError:
                pass
        elif event.ui_element in self.take_buttons:
            try:
                index = self.take_buttons.index(event.ui_element)
                self.take_card_keys.append(CARDS[index])
                self.update_take_cards()
            except ValueError:
                pass
        elif event.ui_element in self.give_card_elements:
            try:
                index = self.give_card_elements.index(event.ui_element)
                self.give_card_keys.pop(index)
                self.update_give_cards()
            except ValueError:
                pass
        elif event.ui_element in self.take_card_elements:
            try:
                index = self.take_card_elements.index(event.ui_element)
                self.take_card_keys.pop(index)
                self.update_take_cards()
            except ValueError:
                pass
        elif event.ui_element == self.trade_with_bank_button:
            self.callback('tradebank', self.give_card_keys, self.take_card_keys)
        elif event.ui_element == self.trade_with_player_button:
            self.callback('tradeplayer', self.give_card_keys, self.take_card_keys)



class PlayerResourceHand:
    def __init__(self):
        self.brick = 0
        self.grain = 0
        self.lumber = 0
        self.ore = 0
        self.wool = 0
        self.totalResources = 0

    def update(self):
        self.totalResources = self.brick + self.grain + self.lumber + self.ore + self.wool


class PlayerDevelopmentHand:
    def __init__(self):
        self.knights = 0
        self.roadBuildings = 0
        self.yearOfPlenty = 0
        self.monopolies = 0
        self.victoryPoints = 0
        self.totalDevelopments = 0

    def update(self):
        self.totalDevelopments = self.knights + self.roadBuildings + self.yearOfPlenty + self.monopolies \
                                 + self.victoryPoints


class EnemyPlayer:
    def __init__(self, turnOrder, name, color, nR, nS, nC, lR, lA, hS, dS, vVP):
        self.turnOrder = turnOrder
        self.name = name
        self.color = color
        self.handSize = hS
        self.developmentSize = dS
        self.visibleVictoryPoints = vVP
        self.numRoads = nR
        self.numSettlements = nS
        self.numCities = nC
        self.longestRoad = lR
        self.largestArmy = lA


class Player:
    def __init__(self, name, color, turnOrder):
        self.color = color
        self.name = name
        self.turnOrder = turnOrder
        self.numRoads = 15
        self.numSettlements = 5
        self.numCities = 4
        self.longestRoad = 0
        self.largestArmy = 0
        self.victoryPoints = 0
        self.resourceHand = PlayerResourceHand()
        self.developmentHand = PlayerDevelopmentHand()
        self.ownedRoads = list()
        self.ownedNodes = list()

    def getNumResources(self):
        return self.resourceHand.totalResources

    def getNumDevelopment(self):
        return self.developmentHand.totalDevelopments

    def getSendToEnemies(self):
        #  toSend = EnemyPlayer(self.turnOrder, self.name, self.color,
        #                      self.numRoads, self.numSettlements, self.numCities,
        #                      self.longestRoad, self.largestArmy)
        toSend = ','.join([self.turnOrder, self.name, self.color, self.numRoads, self.numSettlements, self.numCities,
                           self.longestRoad, self.largestArmy])
        return toSend

    def acquireRoad(self, road):
        self.ownedRoads.append(road)

    def acquireNode(self, node):
        self.ownedNodes.append(node)

    def addResources(self, array):
        self.resourceHand.brick += array[0]
        self.resourceHand.grain += array[1]
        self.resourceHand.lumber += array[2]
        self.resourceHand.ore += array[3]
        self.resourceHand.wool += array[4]
        self.resourceHand.totalResources += array[0] + array[1] + array[2] + array[3] + array[4]



import random
import socket
import time
import sys

from frontiersman.client import ClientHelper


class client:
    def __init__(self, name, address, socket, color):
        self.name = name
        self.address = address
        self.socket = socket
        self.color = color


class server:
    def __init__(self, host, port):

        sep = '\n'

        def dice_roll():
            return (str(random.randint(1, 6)) + ',' + str(random.randint(1, 6)))

        def readfromBuffer(sock):
            totalMessage = ""
            while sep not in totalMessage:
                totalMessage += sock.recv(1).decode('utf-8')
            return totalMessage[:-1]

        clients = []
        Colors = ['red', 'cyan', 'orange', 'blue', 'green', 'pink', 'yellow']
        random.shuffle(Colors)
        ServerSocket = socket.socket()
        try:
            ServerSocket.bind((host, port))
        except socket.error as e:
            print(str(e))

        def sendToAll(message):
            for cli in clients:
                cli.socket.send(str.encode(message + "\n"))

        def sendToAllButOne(message, cli2):
            for cli in clients:
                if (cli != cli2):
                    cli.socket.send(str.encode(message + "\n"))

        print('Waitiing for a Connection..')
        ServerSocket.listen(5)

        Client, address = ServerSocket.accept()
        res = readfromBuffer(Client)
        name = res
        print(name)
        Client.send(str.encode("host\n"))
        clients.append(client(name, address, Client, Colors[0]))
        Colors.pop(0)
        numplayers = readfromBuffer(Client)
        print(int(numplayers))
        for x in range(int(numplayers) - 1):
            Client, address = ServerSocket.accept()
            res = readfromBuffer(Client)
            name = res
            print(name)
            # for cli in clients:
            #    i =0
            #    cli.socket.send(str.encode(name+" connected\n"))
            clients.append(client(name, address, Client, Colors[0]))
            Colors.pop(0)
        for cli in clients:
            sendToAllButOne("enemy," + cli.name + "," + cli.color, cli)
        for i in range(0, len(clients)):
            sendstring = "color,"
            sendstring = sendstring + clients[i].color+','+str(i) + '\n'
            clients[i].socket.send(str.encode(sendstring))
        # board randomizer
        resource_list = ['Wheat'] * 4 + \
                        ['Sheep'] * 4 + \
                        ['Ore'] * 3 + \
                        ['Brick'] * 3 + \
                        ['Wood'] * 4 + \
                        ['Desert'] * 1
        number_list = [2, 12]
        for index in range(3, 12):
            if index == 7:
                pass
            else:
                number_list.append(index)
                number_list.append(index)
        port_list = ['Wheat'] + \
                    ['Sheep'] + \
                    ['Ore'] + \
                    ['Brick'] + \
                    ['Wood'] + \
                    ['None'] * 4
        developmentDeck = ['knight'] * 15 + \
                          ['roadBuilding'] * 2 + \
                          ['yearOfPlenty'] * 2 + \
                          ['monopoly'] * 2 + \
                          ['victoryPoint'] * 5
        random.shuffle(developmentDeck)
        random.shuffle(number_list)
        random.shuffle(resource_list)
        random.shuffle(port_list)
        numberstring = 'board|' + ','.join([str(elem) for elem in number_list]) + '|' + ','.join(
            resource_list) + '|' + ','.join(port_list)
        sendToAll(numberstring)
        for cli in clients:
            cli.socket.send(str.encode("startgui\n"))
            readfromBuffer(cli.socket)
        winner = False
        # setup
        random.shuffle(clients)
        for cli in clients:
            cli.socket.send(str.encode("set\n"))
            coordinates = readfromBuffer(cli.socket)
            sendToAllButOne("set," + coordinates + "," + cli.color, cli)
            message = readfromBuffer(cli.socket)
            sendToAllButOne(message, cli)
            cli.socket.send(str.encode('startroad\n'))
            coordinates = readfromBuffer(cli.socket)
            sendToAllButOne("road," + coordinates + "," + cli.color, cli)
            message = readfromBuffer(cli.socket)
            sendToAllButOne(message, cli)
        for cli in reversed(clients):
            cli.socket.send(str.encode("set\n"))
            coordinates = readfromBuffer(cli.socket)
            sendToAllButOne("set," + coordinates + "," + cli.color, cli)
            message = readfromBuffer(cli.socket)
            sendToAllButOne(message, cli)
            cli.socket.send(str.encode('startroad\n'))
            coordinates = readfromBuffer(cli.socket)
            sendToAllButOne("road," + coordinates + "," + cli.color, cli)
            message = readfromBuffer(cli.socket)
            sendToAllButOne(message, cli)
        for cli in clients:
            cli.socket.send(str.encode('getstart\n'))
            message = readfromBuffer(cli.socket)
            sendToAllButOne(message, cli)
            message = readfromBuffer(cli.socket)
            sendToAllButOne(message, cli)

        while (not winner):
            for cli in clients:
                dice = dice_roll()
                sendToAll('dice,' + dice)
                for cli2 in clients:
                    message = readfromBuffer(cli2.socket)
                    sendToAllButOne(message, cli2)
                    message = readfromBuffer(cli2.socket)
                    sendToAllButOne(message, cli2)
                if (int(dice.split(',')[0]) + int(dice.split(',')[1]) == 7):
                    cli.socket.send(str.encode('robber\n'))
                else:
                    cli.socket.send(str.encode('turn\n'))
                while True:
                    message = readfromBuffer(cli.socket)
                    #print(message)
                    if (message == "end"):
                        break
                    if (message.split(',')[0] == "winner"):
                        #print(message)
                        break
                    elif (message == "dev"):
                        card = developmentDeck.pop(0)
                        cli.socket.send(str.encode(card + '\n'))
                    else:
                        sendToAllButOne(message, cli)
                cli.socket.send(str.encode('notturn\n'))
                # message=readfromBuffer(cli.socket)
                # print(message)
                # sendToAllButOne(message, cli)

        time.sleep(10)
        # game loop

        for cli in clients:
            cli.socket.send(str.encode("quit\n"))
        ServerSocket.close()

def main():
    address='127.0.0.1'
    port=1233
    if(len(sys.argv)==3):
        address=sys.argv[1]
        try:
            port=sys.argv[2]
        except ValueError:
            print("invalid port")
        else:
            server(address, int(port))
    else:
        print("please put the address and port of the server")
    


if __name__ == "__main__":
    host = '127.0.0.1'
    port = 1233
    test = server(host, port)
#     host = sys.argv[1]
#     port = int(sys.argv[2])
#     test = server(host, port)



import math
import os

import pygame


class road_hardware_art:
    @staticmethod
    def generate_road_polygons(road_length, road_ratio):
        # create 3 types of roads, |, /, \
        road_points = []
        road_width = road_length * road_ratio

        road_points.append([
            [-road_width, road_length],
            [road_width, road_length],
            [road_width, -road_length],
            [-road_width, -road_length]
        ])

        angle = math.pi * 5 / 6
        half_angle = math.pi / 2
        end_points = [[road_length * math.cos(angle), road_length * math.sin(angle)],
                      [-1 * road_length * math.cos(angle), -1 * road_length * math.sin(angle)]]
        road_points.append([])

        road_points[1].append([end_points[0][0] - road_width * math.cos(angle + half_angle),
                               end_points[0][1] - road_width * math.sin(angle + half_angle)])
        road_points[1].append([end_points[0][0] - road_width * math.cos(angle - half_angle),
                               end_points[0][1] - road_width * math.sin(angle - half_angle)])
        road_points[1].append([end_points[1][0] - road_width * math.cos(angle - half_angle),
                               end_points[1][1] - road_width * math.sin(angle - half_angle)])
        road_points[1].append([end_points[1][0] - road_width * math.cos(angle + half_angle),
                               end_points[1][1] - road_width * math.sin(angle + half_angle)])

        angle = math.pi / 6
        end_points = [[road_length * math.cos(angle), road_length * math.sin(angle)],
                      [-1 * road_length * math.cos(angle), -1 * road_length * math.sin(angle)]]
        road_points.append([])

        road_points[2].append([end_points[0][0] - road_width * math.cos(angle + half_angle),
                               end_points[0][1] - road_width * math.sin(angle + half_angle)])
        road_points[2].append([end_points[0][0] - road_width * math.cos(angle - half_angle),
                               end_points[0][1] - road_width * math.sin(angle - half_angle)])
        road_points[2].append([end_points[1][0] - road_width * math.cos(angle - half_angle),
                               end_points[1][1] - road_width * math.sin(angle - half_angle)])
        road_points[2].append([end_points[1][0] - road_width * math.cos(angle + half_angle),
                               end_points[1][1] - road_width * math.sin(angle + half_angle)])

        return road_points


class texture_scale:
    @staticmethod
    def scale_resource_tiles(hex_size):

        base_path = os.path.dirname(__file__)

        texture_dictionary = {
            'Desert': pygame.image.load(os.path.join(base_path, '../assets/hexes/desert.png')),
            'Ocean': pygame.image.load(os.path.join(base_path, '../assets/hexes/ocean.png')),
            'Wheat': pygame.image.load(os.path.join(base_path, '../assets/hexes/wheat.png')),
            'Brick': pygame.image.load(os.path.join(base_path, '../assets/hexes/quarry.png')),
            'Ore': pygame.image.load(os.path.join(base_path, '../assets/hexes/mountain.png')),
            'Sheep': pygame.image.load(os.path.join(base_path, '../assets/hexes/plains.png')),
            'Wood': pygame.image.load(os.path.join(base_path, '../assets/hexes/forest.png')),
        }

        for key in texture_dictionary:
            picture = texture_dictionary.get(key)
            texture_dictionary[key] = pygame.transform.scale(picture, (int(hex_size), int(hex_size)))

        return texture_dictionary

    @staticmethod
    def scale_port_tiles(hex_size):

        base_path = os.path.dirname(__file__)

        port_textures = {
            'None': pygame.image.load(os.path.join(base_path, '../assets/default_port.png')),
            'Wheat': pygame.image.load(os.path.join(base_path, '../assets/wheat_port.png')),
            'Brick': pygame.image.load(os.path.join(base_path, '../assets/brick_port.png')),
            'Ore': pygame.image.load(os.path.join(base_path, '../assets/ore_port.png')),
            'Sheep': pygame.image.load(os.path.join(base_path, '../assets/sheep_port.png')),
            'Wood': pygame.image.load(os.path.join(base_path, '../assets/wood_port.png')),
        }

        for key in port_textures:
            picture = port_textures.get(key)
            port_textures[key] = pygame.transform.scale(picture, (int(hex_size), int(hex_size)))

        return port_textures

    @staticmethod
    def scale_cards(card_size):

        base_path = os.path.dirname(__file__)

        CARD_IMAGES = {
            'knight': pygame.image.load(os.path.join(base_path, '../assets/cards/dev/Knight.png')),
            'monopoly': pygame.image.load(os.path.join(base_path, '../assets/cards/dev/Monopoly.png')),
            'roadBuilding': pygame.image.load(os.path.join(base_path, '../assets/cards/dev/Road_Building.png')),
            'yearOfPlenty': pygame.image.load(os.path.join(base_path, '../assets/cards/dev/Year_Of_Plenty.png')),
            'victoryPoint': pygame.image.load(os.path.join(base_path, '../assets/cards/dev/Victory_Point.png')),
            'Brick': pygame.image.load(os.path.join(base_path, '../assets/cards/res/bricks.png')),
            'Ore': pygame.image.load(os.path.join(base_path, '../assets/cards/res/ore.png')),
            'Sheep': pygame.image.load(os.path.join(base_path, '../assets/cards/res/sheep.png')),
            'Wheat': pygame.image.load(os.path.join(base_path, '../assets/cards/res/wheat.png')),
            'Wood': pygame.image.load(os.path.join(base_path, '../assets/cards/res/wood.png'))
        }

        for key in CARD_IMAGES:
            picture = CARD_IMAGES.get(key)
            CARD_IMAGES[key] = pygame.transform.scale(picture, card_size)

        return CARD_IMAGES


class button_resources:
    @staticmethod
    def save_scaled_cards(scaled_cards):
        base_path = os.path.dirname(__file__)

        for key in scaled_cards.keys():
            card = scaled_cards[key]
            pygame.image.save(card, os.path.join(base_path, '../assets/scaled/' + key + '.png'))

            darkened_amount = 10
            darkened_card = card.copy()
            dark = pygame.Surface((card.get_width(), card.get_height()), flags=pygame.SRCALPHA)
            dark.fill((darkened_amount, darkened_amount, darkened_amount, 0))
            darkened_card.blit(dark, (0, 0), special_flags=pygame.BLEND_RGBA_SUB)
            pygame.image.save(darkened_card, os.path.join(base_path, '../assets/scaled/' + key + '_disabled.png'))

            lightened_amount = 10
            lightened_card = card.copy()
            light = pygame.Surface((card.get_width(), card.get_height()), flags=pygame.SRCALPHA)
            light.fill((lightened_amount, lightened_amount, lightened_amount, 0))
            lightened_card.blit(light, (0, 0), special_flags=pygame.BLEND_RGBA_ADD)
            pygame.image.save(lightened_card, os.path.join(base_path, '../assets/scaled/' + key + '_hovered.png'))



import math


# Class defining corners of hexagons
class CornerNode:
    @staticmethod
    def translate_settlement(location, origin, radius, sideLength=3):
        toReturnX = 0
        toReturnY = 0
        edgeDist = (radius / 2) * math.sqrt(3)
        cornerDist = radius

        toReturnX = location[0] * edgeDist
        if (location[0] % 2) == 0:
            toReturnY = (radius / 2) + \
                        (int(location[1] / 2) * (3 * radius)) + \
                        ((location[1] % 2) * radius)
        elif (location[0] % 2) == 1:
            toReturnY = (int(location[1] / 2) * (3 * radius)) + \
                        ((location[1] % 2) * (radius * 2))

        ourXOffset = ((sideLength - 1) * 2) * edgeDist
        ourYOffset = edgeDist + ((sideLength - 1.5) * ((radius / 2) * 3))
        totalXOffset = origin[0] - ourXOffset + 12.5
        totalYOffset = origin[1] - ourYOffset + 12.5

        return int(toReturnX + totalXOffset), int(toReturnY + totalYOffset)

    def __init__(self, r=False, c1=None, c2=None):
        self.real = r
        self.hexs = list()
        self.edges = list()
        self.cord1 = c1
        self.cord2 = c2
        self.available = r
        self.color = None
        self.city = False

    def addHexs(self, h1, h2, h3):
        self.hexs.clear()
        self.hexs.append(h1)
        self.hexs.append(h2)
        self.hexs.append(h3)

    def addEdges(self, e1, e2, e3):
        self.edges.clear()
        self.edges.append(e1)
        self.edges.append(e2)
        self.edges.append(e3)

    def addAll(self, h1, h2, h3, e1, e2, e3):
        self.hexs.clear()
        self.hexs.append(h1)
        self.hexs.append(h2)
        self.hexs.append(h3)
        self.edges.clear()
        self.edges.append(e1)
        self.edges.append(e2)
        self.edges.append(e3)

    def claimPlace(self, color):
        for edge in self.edges:
            if edge is not None:
                if edge.getOpposite(self) is not None:
                    edge.getOpposite(self).adjacentClaimed()
        self.color = color
        self.available = False

    def adjacentClaimed(self):
        self.available = False


class EdgeNode:
    @staticmethod
    def translate_road(location, origin, radius, sideLength=3):
        toReturnX = 0
        toReturnY = 0
        edgeDist = (radius / 2) * math.sqrt(3)
        cornerDist = radius

        toReturnX = location[0] * (edgeDist / 2)
        l1Plus = location[1] + 1
        toReturnY = (radius / 4) \
                    + (location[1] / 2) * (radius * 1.5)

        ourXOffset = ((sideLength - 1) * 2) * edgeDist
        ourYOffset = edgeDist + ((sideLength - 1.5) * ((radius / 2) * 3))
        totalXOffset = origin[0] - ourXOffset + 12.5
        totalYOffset = origin[1] - ourYOffset + 12.5

        return int(toReturnX + totalXOffset), int(toReturnY + totalYOffset)

    def __init__(self, r=False, c1=None, c2=None):
        self.real = r
        self.hexs = list()
        self.corners = list()
        self.cord1 = c1
        self.cord2 = c2
        self.available = r
        self.color = None

    def addHexs(self, h1, h2):
        self.hexs.clear()
        self.hexs.append(h1)
        self.hexs.append(h2)

    def addCorners(self, c1, c2):
        self.corners.append(c1)
        self.corners.append(c2)

    def addAll(self, h1, h2, c1, c2):
        self.hexs.clear()
        self.hexs.append(h1)
        self.hexs.append(h2)
        self.corners.append(c1)
        self.corners.append(c2)

    def getOpposite(self, baseCorner):
        for corner in self.corners:
            if corner != baseCorner and corner is not None:
                return corner
        else:
            print("Error, not a corner not attached to this edge")

    def claim(self, color):
        self.color = color
        self.available = False


class StructureBoard:
    def linkHexes(self, arrayOfHexes):
        cordToX = 0
        cordToY = 0
        cordToZ = 0
        for i in range(0, self.settleColLength):
            for j in range(0, self.settleRowLength):
                if self.settlements[i][j].real:
                    x = self.settlements[i][j].cord1
                    y = self.settlements[i][j].cord2
                    self.settlements[i][j].hexs.clear()
                    for i in range(0, 3):
                        if ((y % 2 == 0) and (x % 2 == 0)) or ((y % 2 == 1) and (x % 2 == 1)):
                            if i == 2:
                                cordToX = int((x - y - self.settleOffset) / 2) - 1
                            else:
                                cordToX = int((x - y - self.settleOffset) / 2)
                            if i == 0:
                                cordToZ = y - self.settleOffset + 1
                            else:
                                cordToZ = y - self.settleOffset + 2
                            cordToY = 0 - cordToX - cordToZ
                        else:
                            if i == 1:
                                cordToX = int((x - y - self.settleOffset) / 2) + 1
                            else:
                                cordToX = int((x - y - self.settleOffset) / 2)
                            if i == 2:
                                cordToZ = y - self.settleOffset + 2
                            else:
                                cordToZ = y - self.settleOffset + 1
                            cordToY = 0 - cordToX - cordToZ
                        try:
                            self.settlements[i][j].hexs.append(arrayOfHexes[cordToX][cordToY])
                        except IndexError:
                            continue

    def __init__(self, sideLength=3, arrayOfHexes=None):
        # Road Setup
        self.roadRowLength = (sideLength * 4) - 1
        self.roadColLength = (((sideLength * 2) - 1) * 4) + 1
        self.roadOffset = (sideLength * 2) - 2
        self.roads = list()

        for i in range(0, self.roadColLength):
            temp = []
            for j in range(0, self.roadRowLength):
                temp.append(EdgeNode())
            self.roads.append(temp)

        for i in range(0, self.roadColLength):
            for j in range(0, self.roadRowLength):
                # taking care of the corners
                if i > ((self.roadColLength - 1) / 2):
                    newCordi = self.roadColLength - 1 - i
                else:
                    newCordi = i

                if j > ((self.roadRowLength - 1) / 2):
                    newCordj = self.roadRowLength - 1 - j
                else:
                    newCordj = j

                if newCordi + newCordj <= self.roadOffset:
                    continue

                # dealing with even rows
                if (j % 2) == 0:
                    if (i % 2) == 1:
                        self.roads[i][j] = EdgeNode(True, i, j)

                # dealing with odd rows
                elif (j % 2) == 1:
                    if (j % 4) == 1:
                        if (i % 4) == 0:
                            self.roads[i][j] = EdgeNode(True, i, j)
                    elif (j % 4) == 3:
                        if (i % 4) == 2:
                            self.roads[i][j] = EdgeNode(True, i, j)

        # Settlement Setup
        self.settleRowLength = sideLength * 2
        self.settleColLength = (sideLength * 4) - 1
        self.settleOffset = sideLength - 1
        self.settlements = list()

        for i in range(0, self.settleColLength):
            temp = []
            for j in range(0, self.settleRowLength):
                temp.append(CornerNode())
            self.settlements.append(temp)

        for i in range(0, self.settleColLength):
            for j in range(0, self.settleRowLength):
                # taking care of the corners
                if i > ((self.settleColLength - 1) / 2):
                    newCordi = self.settleColLength - 1 - i
                else:
                    newCordi = i

                if j > (self.settleRowLength / 2):
                    newCordj = self.settleRowLength - 1 - j
                else:
                    newCordj = j

                if newCordi + newCordj < self.settleOffset:
                    continue

                self.settlements[i][j] = CornerNode(True, i, j)

        for j in range(0, self.roadRowLength):
            for i in range(0, self.roadColLength):
                if self.roads[i][j].real:
                    if (j % 2) == 0:
                        i1 = int(i / 2)
                        j1 = int(j / 2)
                        i2 = int(i / 2) + 1
                        j2 = int(j / 2)
                    else:
                        i1 = int(i / 2)
                        j1 = int(j / 2)
                        i2 = int(i / 2)
                        j2 = int(j / 2) + 1
                    self.roads[i][j].addCorners(self.settlements[i1][j1], self.settlements[i2][j2])

        for j in range(0, self.settleRowLength):
            for i in range(0, self.settleColLength):
                if self.settlements[i][j].real:
                    if ((i % 2) == 0 and (j % 2) == 0) or ((i % 2) == 1 and (j % 2) == 1):
                        i1 = i * 2 - 1
                        if i1 < 0:
                            i1 = None
                        j1 = j * 2
                        i2 = i * 2 + 1
                        if i2 >= self.roadColLength:
                            i2 = None
                        j2 = j * 2
                        i3 = i * 2
                        j3 = j * 2 + 1
                        if j3 >= self.roadRowLength:
                            j3 = None
                    else:
                        i1 = i * 2 - 1
                        if i1 < 0:
                            i1 = None
                        j1 = j * 2
                        i2 = i * 2 + 1
                        if i2 >= self.roadColLength:
                            i2 = None
                        j2 = j * 2
                        i3 = i * 2
                        j3 = j * 2 - 1
                        if j3 < 0:
                            j3 = None
                    insert1 = None
                    insert2 = None
                    insert3 = None

                    if (i1 is not None) and (j1 is not None):
                        insert1 = self.roads[i1][j1]
                    if (i2 is not None) and (j2 is not None):
                        insert2 = self.roads[i2][j2]
                    if (i3 is not None) and (j3 is not None):
                        insert3 = self.roads[i3][j3]

                    self.settlements[i][j].addEdges(insert1, insert2, insert3)

        self.emptySettlementList = list()
        for i in range(0, self.settleColLength):
            for j in range(0, self.settleRowLength):
                if self.settlements[i][j].real:
                    self.emptySettlementList.append(self.settlements[i][j])

        self.emptyRoadList = list()
        for i in range(0, self.roadColLength):
            for j in range(0, self.roadRowLength):
                if self.roads[i][j].real:
                    self.emptyRoadList.append(self.roads[i][j])
        # self.linkHexes(arrayOfHexes)

    def printAllSettlements(self):
        for j in range(0, self.settleRowLength):
            for i in range(0, self.settleColLength):
                print(int(self.settlements[i][j].available), end=" ")
            print()
        print()

        print(len(self.emptySettlementList))
        print()

    def printAllSettlementsArray(self):
        for j in range(0, self.settleRowLength):
            print('[', end="")
            for i in range(0, self.settleColLength):
                print(int(self.settlements[i][j].available), end=", ")
            print('],')
        print()

        print(len(self.emptySettlementList))
        print()

    def printAllRoads(self):
        for j in range(0, self.roadRowLength):
            for i in range(0, self.roadColLength):
                print(int(self.roads[i][j].available), end=" ")
            print()
        print()

        print(len(self.emptyRoadList))
        print()

    def claimInitialSettlement(self, x, y, color):
        toReturn = None
        print("Claimed " + str(x) + " " + str(y))
        print()
        print("Available Roads")
        toReturn = list()
        for settlement in self.emptySettlementList:
            if settlement.cord1 == x and settlement.cord2 == y:
                i = 0
                for edge in settlement.edges:
                    if edge is not None:
                        if edge.available:
                            print("Road " + str(i) + " " + str(edge.cord1) + " " + str(edge.cord2))
                    i += 1
                print()

        toReturn = self.claimSettlement(x, y, color)
        return toReturn

    def claimSettlement(self, x, y, color):
        toReturn = None
        for settlement in self.emptySettlementList:
            if settlement.cord1 == x and settlement.cord2 == y:
                settlement.claimPlace(color)
                toReturn = settlement

        for settlement in self.emptySettlementList:
            if not settlement.available:
                self.emptySettlementList.remove(settlement)

        for settlement in self.emptySettlementList:
            if not settlement.available:
                self.emptySettlementList.remove(settlement)

        for settlement in self.emptySettlementList:
            if not settlement.available:
                self.emptySettlementList.remove(settlement)

        return toReturn

    # def claimInitialRoad(self, x, y, color):

    def claimRoad(self, x, y, color):
        toReturn = None
        for road in self.emptyRoadList:
            if road.cord1 == x and road.cord2 == y:
                road.claim(color)
                toReturn = road

        for road in self.emptyRoadList:
            if not road.available:
                self.emptyRoadList.remove(road)

        for road in self.emptyRoadList:
            if not road.available:
                self.emptyRoadList.remove(road)

        return toReturn

    # def initilizaeHexes(self, hexArray):





import sys

sys.path.insert(0, '../src')
from frontiersman.gameboard.NodeRoads import *


def buildRoadCheck(actor, free=False):
    hand = actor.resourceHand
    if (free and actor.numRoads > 0):
        return True
    if hand.brick < 1 or hand.lumber < 1 or actor.numRoads == 0:
        return False
    else:
        return True


def buildRoadAvailable(actor, free=False):
    toReturn = list()
    if buildRoadCheck(actor, free):
        for edgeBase in actor.ownedRoads:
            for corner in edgeBase.corners:
                for edge in corner.edges:
                    if edge is not None:
                        if edge.available and edge not in toReturn:
                            toReturn.append(edge)
        return toReturn
    else:
        return toReturn


def buildSettlementCheck(actor):
    hand = actor.resourceHand
    if hand.brick < 1 or hand.lumber < 1 or hand.grain < 1 or hand.wool < 1 or actor.numSettlements == 0:
        return False
    else:
        return True


def buildSettlementAvailable(actor):
    toReturn = list()
    if buildSettlementCheck(actor):
        for edgeBase in actor.ownedRoads:
            for corner in edgeBase.corners:
                if corner.available and corner not in toReturn:
                    toReturn.append(corner)
        return toReturn
    else:
        return toReturn


def buildCityCheck(actor):
    hand = actor.resourceHand
    if hand.grain < 2 or hand.ore < 3:  # or actor.numCities == 0 or actor.numSettlements == 5:
        return False
    else:
        return True


def buildCityAvailable(actor):
    toReturn = list()
    if buildCityCheck(actor):
        for corner in actor.ownedNodes:
            if not corner.city:
                toReturn.append(corner)
        return toReturn
    else:
        return toReturn


def buyDevelopmentCheck(actor):
    hand = actor.resourceHand
    if hand.grain < 1 or hand.ore < 1 or hand.wool < 1:
        return False
    else:
        return True


def buyDevelopment(actor, bank):
    if buyDevelopmentCheck(actor):
        '''
        randomCard = random.uniform(0, bank.totalDevelopments)

        if randomCard - bank.knights < 0:
            bank.knights -= 1
            actor.developmentHand.knights += 1
        else:
            randomCard -= bank.knights

        if randomCard - bank.roadBuildings < 0:
            bank.roadBuildings -= 1
            actor.developmentHand.roadBuildings += 1
        else:
            randomCard -= bank.roadBuildings

        if randomCard - bank.yearOfPlenty < 0:
            bank.yearOfPlenty -= 1
            actor.developmentHand.yearOfPlenty += 1
        else:
            randomCard -= bank.yearOfPlenty

        if randomCard - bank.monopolies < 0:
            bank.monopolies -= 1
            actor.developmentHand.monopolies += 1
        else:
            randomCard -= bank.monopolies

        if randomCard - bank.victoryPoints < 0:
            bank.victoryPoints -= 1
            actor.developmentHand.victoryPoints += 1
        else:
            randomCard -= bank.victoryPoints
        '''
        print("check passed")

    else:
        return None


def trade(offeredCards, requestedCards, playerResources, bankResources):
    pass



import io
from setuptools import setup, find_packages

setup(
    name="Frontiersman",
    version="1.0.9",
    author="Brian Snow",
    author_email="snowb@ufl.edu",
    description="Network Multiplayer Board Game",
    install_requires=['pygame>=2.0.0.dev10', 'pygame_gui'],
    long_description=io.open(
        './README.md', 'r', encoding='utf-8').read(),
    url="https://gitlab.com/csi4930-frontiersman/csi4930-frontiersman",
    packages=find_packages(),
    include_package_data = True,
    package_data = {
        '': ['*.png', '*.json'],
    },
    setup_requires=[ "setuptools_git >= 0.3", ],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.8',
    entry_points={
        'console_scripts': [
            'FMClient = frontiersman.ClientMain:main',
            'FMServer = frontiersman.server:main'
        ],
    }
)


import sys

sys.path.insert(0, '../src')
import pygame


class ButtonState:  # credit to https://www.reddit.com/r/pygame/comments/hblj83/circular_button_with_variable_width_and_height/
    def __init__(self, color, size, position, anchor):
        self.rect = pygame.Rect((0, 0), size)
        self.image = pygame.Surface(size, pygame.SRCALPHA)
        self.image.fill((0, 0, 0, 0))
        pygame.draw.ellipse(self.image, color, self.rect)
        setattr(self.rect, anchor, position)

    def draw(self, surface):
        surface.blit(self.image, self.rect)


class CircleButton:
    def __init__(self, normalcolor, hovercolor, size, position, anchor, callback, user_data=None):
        #self.normal = ButtonState(normalcolor, size, position, "center")
        #self.hover = ButtonState(hovercolor, size, position, "center")
        self.normal = ButtonState(normalcolor, size, (position[0] - size[0]/4, position[1] - size[1]/4), "center")
        self.hover = ButtonState(hovercolor, size, (position[0] - size[0]/4, position[1] - size[1]/4), "center")
        self.image = self.normal
        self.callback = callback
        self.user_data = user_data

    def draw(self, surface):
        self.image.draw(surface)

    def on_mousemotion(self, event):
        if self.normal.rect.collidepoint(event.pos):
            pos = event.pos[0] - self.normal.rect.x, event.pos[1] - self.normal.rect.y
            if self.normal.image.get_at(pos).a:
                self.image = self.hover
            else:
                self.image = self.normal
        else:
            self.image = self.normal

    def on_mousebuttondown(self, event):
        if self.hover.rect.collidepoint(event.pos):
            pos = event.pos[0] - self.hover.rect.x, event.pos[1] - self.hover.rect.y
            if self.hover.image.get_at(pos).a:
                self.callback(self.user_data)



#import ctypes
#ctypes.windll.user32.SetProcessDPIAware()
import math
import os
import random
import sys
from typing import Union

import pygame.gfxdraw
import pygame_gui
import pygame_gui as pg_g
from pygame_gui.core import UIElement, IContainerLikeInterface
from pygame_gui.elements import UIWindow, UIImage, UIVerticalScrollBar

os.environ['SDL_AUDIODRIVER'] = 'dsp'
sys.path.insert(0, '../frontiersman')
from frontiersman.gameboard.NodeRoads import *
from frontiersman.gameboard.SettlementButtons import *
from frontiersman.client.Bank import *
from frontiersman.client import ClientHelper
from frontiersman.client import Actions
from frontiersman.client.CardPickerGui import CardPickerGui
from frontiersman.client.TraderGui import TradeGui
#from frontiersman.client.GuiConstants import CARD_SIZE, SPACING

#assuming 1920 x 1080 to be res_full = 100.00

# todo move some of these to GuiConstants.py
RESOLUTION = (1920, 1080)
FRAME_RATE = 60
ACTION_SIZE = (70, 98)
SCALE = 5 / 16
HEX_SIZE = 600 * SCALE
WINDOW_CENTER = (RESOLUTION[0] / 2, RESOLUTION[1] / 2)

#for prototype
FULLSCREEN = False
RESOLUTION = (960, 540)
PROTOTYPE_SCALE = 50/100
ACTION_SIZE = (int(70 * PROTOTYPE_SCALE), int(98 * PROTOTYPE_SCALE))
HEX_SIZE = 600 * SCALE * PROTOTYPE_SCALE
WINDOW_CENTER = (RESOLUTION[0] / 2, RESOLUTION[1] / 2)
SPACING = 1
CARD_SIZE = (35, 45)

COLORS = {
    'red': (127, 0, 0),
    'cyan': (0, 255, 255),
    'orange': (255, 106, 0),
    'blue': (0, 38, 255),
    'green': (0, 153, 15),
    'pink': (255, 0, 110),
    'yellow': (255, 216, 0)
}
INSTRUCTION_TEXT = {
    'display': '',
    'turn': '',
    'road': 'Choose Location for Road',
    'roadfree': 'Choose Location for Road',
    'initialroad': 'Choose Location for Road',
    'set': 'Choose Location for Settlement',
    'initialsettlement': 'Choose Location for Settlement',
    'city': 'Choose Settlement to Upgrade',
    "robber": 'Choose Tile for Robber'
}
DEVELOPMENT_TEXT = {
    'knight': "Move the Robber",
    'roadBuilding': "build 2 free roads",
    'yearOfPlenty': "get 2 of any resource from the bank",
    'monopoly': "Take the resource of your choice from other players",
    'victoryPoint': 'Hidden Victory Point'
}
CARD_IMAGES = ClientHelper.texture_scale.scale_cards(CARD_SIZE)
# ClientHelper.button_resources.save_scaled_cards(CARD_IMAGES)
PORT_TEXTURES = ClientHelper.texture_scale.scale_port_tiles(HEX_SIZE)
TILE_TEXTURES = ClientHelper.texture_scale.scale_resource_tiles(HEX_SIZE)
ROAD_POINTS = ClientHelper.road_hardware_art.generate_road_polygons(HEX_SIZE / 4, 1 / 15)
INFO_WIDTH = 320
BANK_HEIGHT = 100
BANK_CARD_WIDTH = (INFO_WIDTH - 8 * SPACING - BANK_HEIGHT) / 6
BANK_HALF_HEIGHT = (BANK_HEIGHT - 3 * SPACING) / 2
PLAYER_HEIGHT = 150
PLAYER_SECTION_WIDTH = (INFO_WIDTH - 4 * SPACING) / 5
PLAYER_QUARTER_HEIGHT = (PLAYER_HEIGHT - 2 * SPACING) / 4

#prototype
'''
BANK_SCALE = 1.0
INFO_WIDTH = int(320 * BANK_SCALE)
BANK_HEIGHT = int(100 * BANK_SCALE)
BANK_CARD_WIDTH = int((INFO_WIDTH - 8 * SPACING - BANK_HEIGHT) / 6 * BANK_SCALE)
BANK_HALF_HEIGHT = int((BANK_HEIGHT - 3 * SPACING) / 2 * BANK_SCALE)
PLAYER_HEIGHT = int(150 * BANK_SCALE)
PLAYER_SECTION_WIDTH = int((INFO_WIDTH - 4 * SPACING) / 5 * BANK_SCALE)
PLAYER_QUARTER_HEIGHT = int((PLAYER_HEIGHT - 2 * SPACING) / 4 * BANK_SCALE)
'''


class help_image(UIVerticalScrollBar):
    def __init__(self, ui_manager,
                 container: Union[IContainerLikeInterface, None] = None,
                 parent_element: UIElement = None,):
        super().__init__(relative_rect=pygame.Rect((769 - 32 - 2, 0 - 2), (32 + 2, 346)),
                         visible_percentage=.62,
                         manager=ui_manager,
                         container=container,
                         parent_element=parent_element)
        self.HELP_IMAGE = pygame.image.load('../assets/help.png')
        self.size = self.HELP_IMAGE.get_size()
        self.background = pygame.Surface(self.size)
        self.background.fill((0, 0, 0))
        self.visible_percentage = .5
        self.manager = ui_manager
        self.background = pygame.Surface(self.size)
        self.background = self.background.convert()

    def draw(self, surface):
        surface.blit(self.background, (0, 0))
        surface.blit(self.HELP_IMAGE, (0, 0 - int(self.scroll_position * 4)))


class help_window(UIWindow):
    def __init__(self, position, ui_manager):
        self.size = (769 + 28, 400)
        image_size = (737, 802)
        super().__init__(pygame.Rect(position, self.size), ui_manager,
                         window_display_title='Help Window',
                         object_id='#help_window')

        surface_size = self.get_container().get_size()
        self.surface_element = UIImage(pygame.Rect((0, 0),
                                                   surface_size),
                                       pygame.Surface(surface_size).convert(),
                                       manager=ui_manager,
                                       container=self,
                                       parent_element=self)

        self.help_image = help_image(ui_manager, container=self, parent_element=self)

        self.is_active = False

        self.set_blocking(True)

    def update(self, time_delta):
        super().update(time_delta)

        self.help_image.draw(self.surface_element.image)


class Client:
    # for mapping hexagon tiles to x,y
    @staticmethod
    def translate_hex_to_xy(location, origin, radius):
        if location == (0, 0, 0):
            return origin
        x_off = 0
        y_off = 3 / 2 * radius * location[2]
        x_constant = .5 * radius * (3 ** 0.5)
        x_factor = location[0] - location[1]
        x_off = x_factor * x_constant

        return int(origin[0] + x_off), int(origin[1] + y_off)

    @staticmethod
    def quit():
        pygame.event.post(pygame.event.Event(pygame.QUIT))

    @staticmethod
    def add_event(event):
        pygame.event.post(event)

    def __init__(self, board_input):
        self.board_input = board_input
        self.board_action = "display"
        self.last_board_action = "display"
        self.board_updated = False
        self.board_input = board_input
        self.settlementButtons = []
        self.property_list = []
        self.road_list = []
        self.manual_card_list = []
        self.card_hand = []
        self.bank = Bank()
        self.running = False
        self.is_rolling = False
        self.directions_text = 'Temporary Text'

        # todo initialize the rest of these
        self.card_keys = []
        self.cards_updated = False
        self.card_element_list = []
        self.player_element_list = []
        self.dice_image = None
        self.dice_button = None
        self.player = None
        self.enemy_list = []
        self.game_board = None
        self.building_locations = None
        self.font = None
        self.manager = None
        self.clock = None
        self.house_points = None
        self.object_textures = None
        self.board_center = None
        self.resource_hexes = None
        self.port_hexes = None
        self.game_window = None
        self.action_panel = None
        self.grid_size = None
        self.set_butts = None
        self.card_panel = None
        self.info_panel = None
        self.road_butts = None
        self.trade_button = None
        self.road_button = None
        self.settlement_button = None
        self.city_button = None
        self.development_button = None
        self.end_turn_button = None
        self.bank_panel = None
        self.bank_logo = None
        self.wood_logo = None
        self.wood_bank = None
        self.brick_logo = None
        self.brick_bank = None
        self.sheep_logo = None
        self.sheep_bank = None
        self.wheat_logo = None
        self.wheat_bank = None
        self.ore_logo = None
        self.ore_bank = None
        self.dev_logo = None
        self.dev_bank = None
        self.card_picker = None
        self.card_trader = None
        self.help_button = None
        self.help_window = None

        # constants translated to the previously used instance variables
        self.hex_size = HEX_SIZE
        self.port_textures = PORT_TEXTURES
        self.texture_dictionary = TILE_TEXTURES
        self.scale = SCALE * PROTOTYPE_SCALE
        self.card_images = CARD_IMAGES
        self.road_points = ROAD_POINTS

    def set_player(self, player):
        self.player = player

    def board_setup_coordinates_display(self, data):
        self.board_input.put(','.join(data))
        self.board_action = self.last_board_action

    def initial_settlement_buttons(self):
        settlement_buttons = []
        for j in range(0, self.building_locations.settleRowLength):
            for i in range(0, self.building_locations.settleColLength):
                if self.building_locations.settlements[i][j].available:
                    settlement_buttons.append(
                        CircleButton((125, 125, 255), (255, 255, 255), (int(40 * PROTOTYPE_SCALE), int(40 * PROTOTYPE_SCALE)),
                                     CornerNode.translate_settlement((i, j), self.board_center, int(self.hex_size / 2)),
                                     "center", self.board_setup_coordinates_display,
                                     [str(i), str(j)]))
        return settlement_buttons

    def initial_road_buttons(self, prop):
        road_buttons = []
        for edge in prop.edges:
            if edge is not None:
                if edge.real:
                    road_buttons.append(
                        CircleButton((125, 125, 255), (255, 255, 255),
                                     (int(40 * PROTOTYPE_SCALE), int(40 * PROTOTYPE_SCALE)),
                                     EdgeNode.translate_road((edge.cord1, edge.cord2), self.board_center,
                                                             int(self.hex_size / 2)), "center",
                                     self.board_setup_coordinates_display,
                                     [str(edge.cord1), str(edge.cord2)]))
        return road_buttons

    def get_settlement_buttons(self):
        settlement_buttons = []
        list_o_setts = Actions.buildSettlementAvailable(self.player)
        for settle in list_o_setts:
            settlement_buttons.append(
                CircleButton((125, 125, 255), (255, 255, 255), (40, 40),
                             CornerNode.translate_settlement((settle.cord1, settle.cord2), self.board_center,
                                                             int(self.hex_size / 2)),
                             "center", self.board_setup_coordinates_display,
                             ["set", str(settle.cord1), str(settle.cord2)]))
        return settlement_buttons

    def get_city_buttons(self):
        settlement_buttons = []
        list_o_setts = Actions.buildCityAvailable(self.player)
        for settle in list_o_setts:
            settlement_buttons.append(
                CircleButton((125, 125, 255), (255, 255, 255), (40, 40),
                             CornerNode.translate_settlement((settle.cord1, settle.cord2), self.board_center,
                                                             int(self.hex_size / 2)),
                             "center", self.board_setup_coordinates_display,
                             ["city", str(settle.cord1), str(settle.cord2)]))
        return settlement_buttons

    def get_road_buttons(self, free=False):
        road_buttons = []
        list_o_roads = Actions.buildRoadAvailable(self.player, free)
        for edge in list_o_roads:
            road_buttons.append(
                CircleButton((125, 125, 255), (255, 255, 255), (40, 40),
                             EdgeNode.translate_road((edge.cord1, edge.cord2), self.board_center,
                                                     int(self.hex_size / 2)), "center",
                             self.board_setup_coordinates_display,
                             ["road", str(edge.cord1), str(edge.cord2)]))
        return road_buttons

    def get_robber_buttons(self):
        road_buttons = []
        for tile in self.resource_hexes:
            if (not tile.get_robber()):
                loc = self.translate_hex_to_xy(tile.location, self.board_center, int(self.hex_size / 2))
                (loc[0] + int(self.hex_size / 3), loc[1] + int(self.hex_size / 3))
                road_buttons.append(
                    CircleButton((125, 125, 255), (255, 255, 255), (40, 40),
                                 (loc[0] + int(self.hex_size / 3), loc[1] + int(self.hex_size / 3)), "center",
                                 self.board_setup_coordinates_display,
                                 ["robber", str(tile.location[0]), str(tile.location[1])]))
        return road_buttons

    def set_cards(self, card_keys):
        self.card_keys = card_keys
        self.cards_updated = True

    def update_cards(self):
        for card in self.card_element_list:
            card.kill()

        self.card_element_list = []
        if len(self.card_keys) > 0:
            area_width = self.card_panel.relative_rect.width - CARD_SIZE[0] - 8
            if len(self.card_keys) == 1:
                offset = 0
            else:
                offset = min(area_width / (len(self.card_keys) - 1), CARD_SIZE[0] + SPACING)

            start = (area_width - offset * (len(self.card_keys) - 1)) / 2
            self.card_keys.sort()
            for i, key in enumerate(self.card_keys):
                if key in DEVELOPMENT_TEXT.keys():
                    self.card_element_list.append(pg_g.elements.UIButton(
                        relative_rect=pygame.Rect((start + i * offset, 0), CARD_SIZE),
                        text="",
                        manager=self.manager,
                        tool_tip_text=DEVELOPMENT_TEXT[key],
                        container=self.card_panel,
                        object_id='#' + key,
                    ))
                else:
                    self.card_element_list.append(pg_g.elements.UIButton(
                        relative_rect=pygame.Rect((start + i * offset, 0), CARD_SIZE),
                        text="",
                        manager=self.manager,
                        container=self.card_panel,
                        object_id='#' + key,
                    ))

    # todo figure out how to make scrollable
    def set_player_info(self, curr_player, enemy_players):
        for panel in self.player_element_list:
            panel.kill()

        self.player_element_list = []

        curr_player_info = pg_g.elements.UIPanel(
            relative_rect=pygame.Rect(0, self.bank_panel.relative_rect.bottom + SPACING, INFO_WIDTH - 2 * SPACING,
                                      PLAYER_HEIGHT + SPACING),
            container=self.info_panel,
            starting_layer_height=1,
            manager=self.manager
        )
        name = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(SPACING, SPACING, PLAYER_SECTION_WIDTH * 2 + SPACING, PLAYER_QUARTER_HEIGHT),
            container=curr_player_info,
            html_text=curr_player.name,
            manager=self.manager,
        )
        hand_logo = pg_g.elements.UITextBox(  # todo replace with logo
            relative_rect=pygame.Rect(name.relative_rect.left, name.relative_rect.bottom, PLAYER_SECTION_WIDTH,
                                      2 * PLAYER_QUARTER_HEIGHT),
            container=curr_player_info,
            html_text="Res",
            manager=self.manager,
        )
        hand_value = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(hand_logo.relative_rect.left, hand_logo.relative_rect.bottom,
                                      PLAYER_SECTION_WIDTH, PLAYER_QUARTER_HEIGHT),
            container=curr_player_info,
            html_text=str(curr_player.resourceHand.totalResources),
            manager=self.manager,
        )
        dev_logo = pg_g.elements.UITextBox(  # todo replace with logo
            relative_rect=pygame.Rect(hand_logo.relative_rect.right, name.relative_rect.bottom, PLAYER_SECTION_WIDTH,
                                      2 * PLAYER_QUARTER_HEIGHT),
            container=curr_player_info,
            html_text="Dev",
            manager=self.manager,
        )
        dev_value = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(dev_logo.relative_rect.left, dev_logo.relative_rect.bottom, PLAYER_SECTION_WIDTH,
                                      PLAYER_QUARTER_HEIGHT),
            container=curr_player_info,
            html_text=str(curr_player.developmentHand.totalDevelopments),
            manager=self.manager,
        )

        vp_logo = pg_g.elements.UITextBox(  # todo replace with logo
            relative_rect=pygame.Rect(name.relative_rect.right, name.relative_rect.top, PLAYER_SECTION_WIDTH,
                                      PLAYER_QUARTER_HEIGHT),
            container=curr_player_info,
            html_text="VP",
            manager=self.manager,
        )
        if curr_player.hiddenVictoryPoints != 0:
            vp_value = pg_g.elements.UITextBox(
                relative_rect=pygame.Rect(vp_logo.relative_rect.left, vp_logo.relative_rect.bottom,
                                          PLAYER_SECTION_WIDTH, PLAYER_QUARTER_HEIGHT),
                container=curr_player_info,
                html_text=str(curr_player.victoryPoints) + '(' + str(
                    curr_player.victoryPoints + curr_player.hiddenVictoryPoints) + ')',
                manager=self.manager,
            )
        else:
            vp_value = pg_g.elements.UITextBox(
                relative_rect=pygame.Rect(vp_logo.relative_rect.left, vp_logo.relative_rect.bottom,
                                          PLAYER_SECTION_WIDTH, PLAYER_QUARTER_HEIGHT),
                container=curr_player_info,
                html_text=str(curr_player.victoryPoints),
                manager=self.manager,
            )

        roads_logo = pg_g.elements.UITextBox(  # todo replace with logo
            relative_rect=pygame.Rect(vp_value.relative_rect.left, vp_value.relative_rect.bottom, PLAYER_SECTION_WIDTH,
                                      PLAYER_QUARTER_HEIGHT),
            container=curr_player_info,
            html_text="#R",
            manager=self.manager,
        )
        roads_value = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(roads_logo.relative_rect.left, roads_logo.relative_rect.bottom,
                                      PLAYER_SECTION_WIDTH, PLAYER_QUARTER_HEIGHT),
            container=curr_player_info,
            html_text=str(curr_player.numRoads),
            manager=self.manager,
        )

        settlements_logo = pg_g.elements.UITextBox(  # todo replace with logo
            relative_rect=pygame.Rect(vp_logo.relative_rect.right, vp_logo.relative_rect.top, PLAYER_SECTION_WIDTH,
                                      PLAYER_QUARTER_HEIGHT),
            container=curr_player_info,
            html_text="#S",
            manager=self.manager,
        )
        settlements_value = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(settlements_logo.relative_rect.left, settlements_logo.relative_rect.bottom,
                                      PLAYER_SECTION_WIDTH, PLAYER_QUARTER_HEIGHT),
            container=curr_player_info,
            html_text=str(curr_player.numSettlements),
            manager=self.manager,
        )
        cities_logo = pg_g.elements.UITextBox(  # todo replace with logo
            relative_rect=pygame.Rect(settlements_value.relative_rect.left, settlements_value.relative_rect.bottom,
                                      PLAYER_SECTION_WIDTH, PLAYER_QUARTER_HEIGHT),
            container=curr_player_info,
            html_text="#C",
            manager=self.manager,
        )
        cities_value = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(cities_logo.relative_rect.left, cities_logo.relative_rect.bottom,
                                      PLAYER_SECTION_WIDTH, PLAYER_QUARTER_HEIGHT),
            container=curr_player_info,
            html_text=str(curr_player.numCities),
            manager=self.manager,
        )

        longest_road_logo = pg_g.elements.UITextBox(  # todo replace with logo
            relative_rect=pygame.Rect(settlements_logo.relative_rect.right, settlements_logo.relative_rect.top,
                                      PLAYER_SECTION_WIDTH, PLAYER_QUARTER_HEIGHT),
            container=curr_player_info,
            html_text="LR",
            manager=self.manager,
        )
        longest_road_value = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(longest_road_logo.relative_rect.left, longest_road_logo.relative_rect.bottom,
                                      PLAYER_SECTION_WIDTH, PLAYER_QUARTER_HEIGHT),
            container=curr_player_info,
            html_text=str(curr_player.longestRoad),
            manager=self.manager,
        )
        largest_army_logo = pg_g.elements.UITextBox(  # todo replace with logo
            relative_rect=pygame.Rect(longest_road_value.relative_rect.left, longest_road_value.relative_rect.bottom,
                                      PLAYER_SECTION_WIDTH, PLAYER_QUARTER_HEIGHT),
            container=curr_player_info,
            html_text="LA",
            manager=self.manager,
        )
        largest_army_value = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(largest_army_logo.relative_rect.left, largest_army_logo.relative_rect.bottom,
                                      PLAYER_SECTION_WIDTH, PLAYER_QUARTER_HEIGHT),
            container=curr_player_info,
            html_text=str(curr_player.largestArmy),
            manager=self.manager,
        )

        self.player_element_list.append(curr_player_info)

        for enemy_player in enemy_players:
            enemy_player_info = pg_g.elements.UIPanel(
                relative_rect=pygame.Rect(0, self.player_element_list[-1].relative_rect.bottom + SPACING,
                                          INFO_WIDTH - 2 * SPACING, PLAYER_HEIGHT + SPACING),
                container=self.info_panel,
                starting_layer_height=1,
                manager=self.manager
            )
            name = pg_g.elements.UITextBox(
                relative_rect=pygame.Rect(SPACING, SPACING, PLAYER_SECTION_WIDTH * 2 + SPACING, PLAYER_QUARTER_HEIGHT),
                container=enemy_player_info,
                html_text=enemy_player.name,
                manager=self.manager,
            )

            hand_logo = pg_g.elements.UITextBox(  # todo replace with logo
                relative_rect=pygame.Rect(name.relative_rect.left, name.relative_rect.bottom, PLAYER_SECTION_WIDTH,
                                          2 * PLAYER_QUARTER_HEIGHT),
                container=enemy_player_info,
                html_text="Res",
                manager=self.manager,
            )
            hand_value = pg_g.elements.UITextBox(
                relative_rect=pygame.Rect(hand_logo.relative_rect.left, hand_logo.relative_rect.bottom,
                                          PLAYER_SECTION_WIDTH, PLAYER_QUARTER_HEIGHT),
                container=enemy_player_info,
                html_text=str(enemy_player.handSize),
                manager=self.manager,
            )
            dev_logo = pg_g.elements.UITextBox(  # todo replace with logo
                relative_rect=pygame.Rect(hand_logo.relative_rect.right, name.relative_rect.bottom,
                                          PLAYER_SECTION_WIDTH, 2 * PLAYER_QUARTER_HEIGHT),
                container=enemy_player_info,
                html_text="Dev",
                manager=self.manager,
            )
            dev_value = pg_g.elements.UITextBox(
                relative_rect=pygame.Rect(dev_logo.relative_rect.left, dev_logo.relative_rect.bottom,
                                          PLAYER_SECTION_WIDTH, PLAYER_QUARTER_HEIGHT),
                container=enemy_player_info,
                html_text=str(enemy_player.developmentSize),
                manager=self.manager,
            )

            vp_logo = pg_g.elements.UITextBox(  # todo replace with logo
                relative_rect=pygame.Rect(name.relative_rect.right, name.relative_rect.top, PLAYER_SECTION_WIDTH,
                                          PLAYER_QUARTER_HEIGHT),
                container=enemy_player_info,
                html_text="VP",
                manager=self.manager,
            )
            vp_value = pg_g.elements.UITextBox(
                relative_rect=pygame.Rect(vp_logo.relative_rect.left, vp_logo.relative_rect.bottom,
                                          PLAYER_SECTION_WIDTH, PLAYER_QUARTER_HEIGHT),
                container=enemy_player_info,
                html_text=str(enemy_player.visibleVictoryPoints),
                manager=self.manager,
            )

            roads_logo = pg_g.elements.UITextBox(  # todo replace with logo
                relative_rect=pygame.Rect(vp_value.relative_rect.left, vp_value.relative_rect.bottom,
                                          PLAYER_SECTION_WIDTH, PLAYER_QUARTER_HEIGHT),
                container=enemy_player_info,
                html_text="#R",
                manager=self.manager,
            )
            roads_value = pg_g.elements.UITextBox(
                relative_rect=pygame.Rect(roads_logo.relative_rect.left, roads_logo.relative_rect.bottom,
                                          PLAYER_SECTION_WIDTH, PLAYER_QUARTER_HEIGHT),
                container=enemy_player_info,
                html_text=str(enemy_player.numRoads),
                manager=self.manager,
            )

            settlements_logo = pg_g.elements.UITextBox(  # todo replace with logo
                relative_rect=pygame.Rect(vp_logo.relative_rect.right, vp_logo.relative_rect.top, PLAYER_SECTION_WIDTH,
                                          PLAYER_QUARTER_HEIGHT),
                container=enemy_player_info,
                html_text="#S",
                manager=self.manager,
            )
            settlements_value = pg_g.elements.UITextBox(
                relative_rect=pygame.Rect(settlements_logo.relative_rect.left, settlements_logo.relative_rect.bottom,
                                          PLAYER_SECTION_WIDTH, PLAYER_QUARTER_HEIGHT),
                container=enemy_player_info,
                html_text=str(enemy_player.numSettlements),
                manager=self.manager,
            )
            cities_logo = pg_g.elements.UITextBox(  # todo replace with logo
                relative_rect=pygame.Rect(settlements_value.relative_rect.left, settlements_value.relative_rect.bottom,
                                          PLAYER_SECTION_WIDTH, PLAYER_QUARTER_HEIGHT),
                container=enemy_player_info,
                html_text="#C",
                manager=self.manager,
            )
            cities_value = pg_g.elements.UITextBox(
                relative_rect=pygame.Rect(cities_logo.relative_rect.left, cities_logo.relative_rect.bottom,
                                          PLAYER_SECTION_WIDTH, PLAYER_QUARTER_HEIGHT),
                container=enemy_player_info,
                html_text=str(enemy_player.numCities),
                manager=self.manager,
            )

            longest_road_logo = pg_g.elements.UITextBox(  # todo replace with logo
                relative_rect=pygame.Rect(settlements_logo.relative_rect.right, settlements_logo.relative_rect.top,
                                          PLAYER_SECTION_WIDTH, PLAYER_QUARTER_HEIGHT),
                container=enemy_player_info,
                html_text="LR",
                manager=self.manager,
            )
            longest_road_value = pg_g.elements.UITextBox(
                relative_rect=pygame.Rect(longest_road_logo.relative_rect.left, longest_road_logo.relative_rect.bottom,
                                          PLAYER_SECTION_WIDTH, PLAYER_QUARTER_HEIGHT),
                container=enemy_player_info,
                html_text=str(enemy_player.longestRoad),
                manager=self.manager,
            )
            largest_army_logo = pg_g.elements.UITextBox(  # todo replace with logo
                relative_rect=pygame.Rect(longest_road_value.relative_rect.left,
                                          longest_road_value.relative_rect.bottom, PLAYER_SECTION_WIDTH,
                                          PLAYER_QUARTER_HEIGHT),
                container=enemy_player_info,
                html_text="LA",
                manager=self.manager,
            )
            largest_army_value = pg_g.elements.UITextBox(
                relative_rect=pygame.Rect(largest_army_logo.relative_rect.left, largest_army_logo.relative_rect.bottom,
                                          PLAYER_SECTION_WIDTH, PLAYER_QUARTER_HEIGHT),
                container=enemy_player_info,
                html_text=str(enemy_player.largestArmy),
                manager=self.manager,
            )

            self.player_element_list.append(enemy_player_info)

    @staticmethod
    def translate_from_3d_tile(coordinates):
        center = [5, 2.5]
        y = coordinates[2]
        x = coordinates[0] - coordinates[1]
        array = [[center[0] + x, int(center[1] + y + .5)], [center[0] + x - 1, int(center[1] + y + .5)],
                 [center[0] + x + 1, int(center[1] + y + .5)], [center[0] + x, int(center[1] + y - .5)],
                 [center[0] + x - 1, int(center[1] + y - .5)], [center[0] + x + 1, int(center[1] + y - .5)]]
        return array

    @staticmethod
    def translate_from_2d_tile(coordinates):
        center = [5, 2.5]
        y = 1 - coordinates[0] - coordinates[1]
        x = coordinates[0] - coordinates[1]
        array = [[center[0] + x, int(center[1] + y + .5)], [center[0] + x - 1, int(center[1] + y + .5)],
                 [center[0] + x + 1, int(center[1] + y + .5)], [center[0] + x, int(center[1] + y - .5)],
                 [center[0] + x - 1, int(center[1] + y - .5)], [center[0] + x + 1, int(center[1] + y - .5)]]
        return array

    @staticmethod
    def get_tiles_from_vertex(coordinates):
        x = coordinates[0]
        y = coordinates[1]

        tile_array = []

        location_list = [
            [0, 2, -2],
            [1, 1, -2],
            [2, 0, -2],
            [-1, 2, -1],
            [0, 1, -1],
            [1, 0, -1],
            [2, -1, -1],
            [-2, 2, 0],
            [-1, 1, 0],
            [0, 0, 0],
            [1, -1, 0],
            [2, -2, 0],
            [-2, 1, 1],
            [-1, 0, 1],
            [0, -1, 1],
            [1, -2, 1],
            [-2, 0, 2],
            [-1, -1, 2],
            [0, -2, 2]
        ]
        translate_list = []
        for tile in location_list:
            for point in Client.translate_from_3d_tile(tile):
                if point == [x, y]:
                    tile_array.append(tile)

        return tile_array

    def process_dice(self, roll):
        cards_to_get = []
        if roll == 7:
            pass  # todo remove cards
        else:
            for prop in self.player.ownedNodes:
                for tile_coordinate in Client.get_tiles_from_vertex([prop.cord1, prop.cord2]):
                    tile = self.game_board.get_tile(tile_coordinate)
                    if tile.number == roll and not tile.get_robber():
                        if prop.city:
                            cards_to_get.append(tile.get_resource())
                            cards_to_get.append(tile.get_resource())
                        else:
                            cards_to_get.append(tile.get_resource())
        self.get_cards(cards_to_get)

    def pay_cards(self, res):
        for r in range(0, res[0]):
            self.card_hand.remove('Brick')
        for r in range(0, res[1]):
            self.card_hand.remove('Wheat')
        for r in range(0, res[2]):
            self.card_hand.remove('Wood')
        for r in range(0, res[3]):
            self.card_hand.remove('Ore')
        for r in range(0, res[4]):
            self.card_hand.remove('Sheep')
        self.set_cards(self.card_hand)
        self.bank.update(res)
        array2 = [0 - x for x in res]
        self.player.addResources(array2)
        self.set_board_updated(True)
        self.board_input.put('bank,' + ','.join([str(x) for x in array2]))

    def trade_cards(self, trade, get):
        array1 = [0, 0, 0, 0, 0]
        for r in trade:
            if r == 'Brick':
                array1[0] -= 1
                self.card_hand.remove('Brick')
            elif r == 'Wheat':
                array1[1] -= 1
                self.card_hand.remove('Wheat')
            elif r == 'Wood':
                array1[2] -= 1
                self.card_hand.remove('Wood')
            elif r == 'Ore':
                array1[3] -= 1
                self.card_hand.remove('Ore')
            elif r == 'Sheep':
                array1[4] -= 1
                self.card_hand.remove('Sheep')
        for r in get:
            if r == 'Brick':
                array1[0] += 1
                self.card_hand.append('Brick')
            elif r == 'Wheat':
                array1[1] += 1
                self.card_hand.append('Wheat')
            elif r == 'Wood':
                array1[2] += 1
                self.card_hand.append('Wood')
            elif r == 'Ore':
                array1[3] += 1
                self.card_hand.append('Ore')
            elif r == 'Sheep':
                array1[4] += 1
                self.card_hand.append('Sheep')
        self.set_cards(self.card_hand)
        self.player.addResources(array1)
        array2 = [0 - x for x in array1]
        self.bank.update(array2)
        self.set_board_updated(True)
        self.board_input.put('bank,' + ','.join([str(x) for x in array2]))

    def get_dev_card(self, card):
        self.bank.get_dev_card()
        self.player.developmentHand.add_card(card)
        self.card_hand.append(card)
        self.set_cards(self.card_hand)
        self.set_board_updated(True)

    def get_cards(self, res):
        array1 = [0, 0, 0, 0, 0]
        for r in res:
            if r == 'Brick':
                array1[0] += 1
            elif r == 'Wheat':
                array1[1] += 1
            elif r == 'Wood':
                array1[2] += 1
            elif r == 'Ore':
                array1[3] += 1
            elif r == 'Sheep':
                array1[4] += 1
        self.card_hand += res
        self.set_cards(self.card_hand)
        self.player.addResources(array1)
        array2 = [0 - x for x in array1]
        self.bank.update(array2)
        self.set_board_updated(True)
        self.board_input.put('bank,' + ','.join([str(x) for x in array2]))

    def get_monopoly(self, r):
        res = []
        array1 = [0, 0, 0, 0, 0]
        if r == 'Brick':
            for x in range(0, 19 - self.bank.brick - self.player.resourceHand.brick):
                res.append(r)
                array1[0] += 1
        elif r == 'Wheat':
            for x in range(0, 19 - self.bank.grain - self.player.resourceHand.grain):
                res.append(r)
                array1[1] += 1
        elif r == 'Wood':
            for x in range(0, 19 - self.bank.lumber - self.player.resourceHand.lumber):
                res.append(r)
                array1[2] += 1
        elif r == 'Ore':
            for x in range(0, 19 - self.bank.ore - self.player.resourceHand.ore):
                res.append(r)
                array1[3] += 1
        elif r == 'Sheep':
            for x in range(0, 19 - self.bank.sheep - self.player.resourceHand.wool):
                res.append(r)
                array1[4] += 1
        self.card_hand += res
        self.set_cards(self.card_hand)
        self.player.addResources(array1)
        self.set_board_updated(True)
        self.board_input.put('monopoly,' + r)

    def pay_monopoly(self, r):
        res = []
        array1 = [0, 0, 0, 0, 0]
        if r == 'Brick':
            for x in range(0, self.player.resourceHand.brick):
                self.card_hand.remove(r)
                array1[0] -= 1
        elif r == 'Wheat':
            for x in range(0, self.player.resourceHand.grain):
                self.card_hand.remove(r)
                self.card_hand.remove(r)
                array1[1] -= 1
        elif r == 'Wood':
            for x in range(0, self.player.resourceHand.lumber):
                self.card_hand.remove(r)
                array1[2] -= 1
        elif r == 'Ore':
            for x in range(0, self.player.resourceHand.ore):
                self.card_hand.remove(r)
                array1[3] -= 1
        elif r == 'Sheep':
            for x in range(0, self.player.resourceHand.wool):
                self.card_hand.remove(r)
                array1[4] -= 1
        self.set_cards(self.card_hand)
        self.player.addResources(array1)
        self.set_board_updated(True)

    def start_resources(self):
        last = self.player.ownedNodes[-1]
        array = Client.get_tiles_from_vertex([last.cord1, last.cord2])
        res = []
        for h in array:
            if self.game_board.get_tile(h).get_resource() != 'Desert':
                res.append(self.game_board.get_tile(h).get_resource())
        self.get_cards(res)

    def set_board_action(self, bool1, last_action=None):
        if (self.board_action != bool1):
            self.board_updated = True
        if last_action == None:
            self.last_board_action = self.board_action
        else:
            self.last_board_action = last_action
        self.board_action = bool1

    def set_board_updated(self, bool1):
        self.board_updated = bool1

    def render_dice(self, rolls):
        size = int(80 * PROTOTYPE_SCALE)
        width = len(rolls) * (2 * SPACING + size) - 2 * SPACING
        height = size

        dice_surface = pygame.Surface((width, height), pygame.SRCALPHA)
        dice_surface.fill((0, 0, 0, 0))

        for i, roll in enumerate(rolls):
            x = i * (2 * SPACING + size) - 2 * SPACING
            y = 0
            pygame.draw.rect(dice_surface, (255, 255, 255), pygame.Rect(x, y, size, size))
            text_surface = self.font.render(str(roll), True, (0, 0, 0))
            dice_surface.blit(text_surface,
                              (x + (size - text_surface.get_width()) / 2, y + (size - text_surface.get_height()) / 2))

        return dice_surface

    def display_dice(self, rolls):
        if self.dice_image:
            self.dice_image.kill()

        dice_render = self.render_dice(rolls)
        dice_rect = pygame.Rect(self.info_panel.relative_rect.left - dice_render.get_width() - SPACING,
                                self.action_panel.relative_rect.top - dice_render.get_height() - SPACING,
                                dice_render.get_width(),
                                dice_render.get_height())

        if self.dice_button is None:
            self.dice_button = pg_g.elements.UIButton(
                relative_rect=dice_rect,
                manager=self.manager,
                text="",
                anchors={'left': 'right',
                         'right': 'right',
                         'top': 'bottom',
                         'bottom': 'bottom'},
            )

        self.dice_image = pg_g.elements.UIImage(
            relative_rect=dice_rect,
            manager=self.manager,
            image_surface=dice_render,
            anchors={'left': 'right',
                     'right': 'right',
                     'top': 'bottom',
                     'bottom': 'bottom'}
        )

    def initialize_gui(self):

        pygame.init()
        pygame.display.set_caption('Client')
        os.chdir(os.path.dirname(__file__))
        font_size = int(45 * PROTOTYPE_SCALE)
        self.font = pygame.font.SysFont('Comic Sans MS', font_size)

        font_size = int(45 * PROTOTYPE_SCALE)
        self.font_bank = pygame.font.SysFont('Comic Sans MS', font_size)

        screenstring = "localhost:0."+self.player.screen_num
        os.environ["DISPLAY"] = screenstring
        os.environ['SDL_VIDEODRIVER'] = 'x11'
        if FULLSCREEN:
            self.game_window = pygame.display.set_mode(RESOLUTION, pygame.FULLSCREEN)
        else:
            self.game_window = pygame.display.set_mode(RESOLUTION)

        cursor = pygame.cursors.arrow
        pygame.mouse.set_cursor(*cursor)

        self.manager = pg_g.UIManager(RESOLUTION, '../themes/card_theme.json')
        self.clock = pygame.time.Clock()

        # creates settlement polygon
        house_width = 120 * self.scale / 2
        house_height = -135 * self.scale / 2
        self.house_width = 120 * self.scale / 2
        self.house_height = -135 * self.scale / 2
        self.house_points = [
            [-house_width, -house_height],
            [-house_width, 0],
            [0, house_height],
            [house_width, 0],
            [house_width, -house_height]
        ]

        # creates city polygon
        city_width = 120 * self.scale / 2
        city_height = -120 * self.scale / 2

        self.city_width = 120 * self.scale / 2
        self.city_height = -120 * self.scale / 2
        difference = -5

        self.city_points = [
            [-city_width, city_height],
            [-city_width / 2, city_height * 5 / 4],
            [0, city_height],
            [0, 0 + difference],
            [city_width, 0 + difference],
            [city_width, -city_height],
            [-city_width, -city_height]
        ]

        base_path = os.path.dirname(__file__)
        self.object_textures = {
            'Board': pygame.image.load('../assets/gameboard_grid_v4.png')
        }

        self.board_center = (
            RESOLUTION[0] / 2 - int(self.hex_size / 2), RESOLUTION[1] / 2 - int(self.hex_size / 2) - CARD_SIZE[1] / 2)
        #prototype
        self.board_center = (
            RESOLUTION[0] / 2 - int(self.hex_size / 2) - 150, RESOLUTION[1] / 2 - int(self.hex_size / 2) - CARD_SIZE[1] / 2)

        self.resource_hexes = self.game_board.land_list
        self.port_hexes = self.game_board.port_list

        # game grid
        self.object_textures['Board'] = pygame.transform.scale(self.object_textures.get('Board'),
                                                               (int(3600 * self.scale), int(3600 * self.scale)))
        self.grid_size = self.object_textures.get('Board').get_size()
        self.set_butts = self.initial_settlement_buttons()
        self.road_butts = []

        action_panel_layout = pygame.Rect(0, 0, 6 * ACTION_SIZE[0] + 7 * SPACING, ACTION_SIZE[1] + 2 * SPACING)
        action_panel_layout.bottomright = (-SPACING, -SPACING)
        self.action_panel = pg_g.elements.UIPanel(relative_rect=action_panel_layout,
                                                  starting_layer_height=1,
                                                  manager=self.manager,
                                                  anchors={'left': 'right',
                                                           'right': 'right',
                                                           'top': 'bottom',
                                                           'bottom': 'bottom'}
                                                  )

        self.trade_button = pg_g.elements.UIButton(
            relative_rect=pygame.Rect((0 * ACTION_SIZE[0] + 0 * SPACING, 0), ACTION_SIZE),
            container=self.action_panel,
            manager=self.manager,
            text="TRADE",
            tool_tip_text="Trade resources with the bank")

        self.road_button = pg_g.elements.UIButton(
            relative_rect=pygame.Rect((1 * ACTION_SIZE[0] + 1 * SPACING, 0), ACTION_SIZE),
            container=self.action_panel,
            manager=self.manager,
            text="ROAD")

        self.settlement_button = pg_g.elements.UIButton(
            relative_rect=pygame.Rect((2 * ACTION_SIZE[0] + 2 * SPACING, 0), ACTION_SIZE),
            container=self.action_panel,
            manager=self.manager,
            text="SETTLE")

        self.city_button = pg_g.elements.UIButton(
            relative_rect=pygame.Rect((3 * ACTION_SIZE[0] + 3 * SPACING, 0), ACTION_SIZE),
            container=self.action_panel,
            manager=self.manager,
            text="CITY")

        self.development_button = pg_g.elements.UIButton(
            relative_rect=pygame.Rect((4 * ACTION_SIZE[0] + 4 * SPACING, 0), ACTION_SIZE),
            container=self.action_panel,
            manager=self.manager,
            text="DEV CARD")

        self.end_turn_button = pg_g.elements.UIButton(
            relative_rect=pygame.Rect((5 * ACTION_SIZE[0] + 5 * SPACING, 0), ACTION_SIZE),
            container=self.action_panel,
            manager=self.manager,
            text="END",
        )

        card_panel_layout = pygame.Rect(0, 0, RESOLUTION[0] - action_panel_layout.width - 2 * SPACING,
                                        CARD_SIZE[1] + 2 * SPACING)
        card_panel_layout.bottomleft = (SPACING, -SPACING)
        self.card_panel = pg_g.elements.UIPanel(relative_rect=card_panel_layout,
                                                starting_layer_height=1,
                                                manager=self.manager,
                                                anchors={'left': 'left',
                                                         'right': 'right',
                                                         'top': 'bottom',
                                                         'bottom': 'bottom'}
                                                )

        info_panel_layout = pygame.Rect(0, 0, INFO_WIDTH, RESOLUTION[1] - action_panel_layout.height - 2 * SPACING)
        info_panel_layout.topright = (-SPACING, SPACING)
        self.info_panel = pg_g.elements.UIPanel(relative_rect=info_panel_layout,
                                                starting_layer_height=1,
                                                manager=self.manager,
                                                anchors={'left': 'right',
                                                         'right': 'right',
                                                         'top': 'top',
                                                         'bottom': 'top'}
                                                )

        self.bank_panel = pg_g.elements.UIPanel(
            relative_rect=pygame.Rect(0, 0, INFO_WIDTH - 2 * SPACING, BANK_HEIGHT + SPACING),
            container=self.info_panel,
            starting_layer_height=1,
            manager=self.manager)

        self.bank_logo = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(SPACING, SPACING, BANK_HEIGHT - 2 * SPACING, BANK_HEIGHT - 2 * SPACING),
            container=self.bank_panel,
            manager=self.manager,
            html_text="Bank"
        )

        self.wood_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(self.bank_logo.relative_rect.right + SPACING, SPACING, BANK_CARD_WIDTH,
                                      BANK_HALF_HEIGHT),
            image_surface=self.card_images["Wood"],
            manager=self.manager,
            container=self.bank_panel)

        self.brick_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(self.wood_logo.relative_rect.right + SPACING, SPACING, BANK_CARD_WIDTH,
                                      BANK_HALF_HEIGHT),
            image_surface=self.card_images["Brick"],
            manager=self.manager,
            container=self.bank_panel)

        self.sheep_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(self.brick_logo.relative_rect.right + SPACING, SPACING, BANK_CARD_WIDTH,
                                      BANK_HALF_HEIGHT),
            image_surface=self.card_images["Sheep"],
            manager=self.manager,
            container=self.bank_panel)

        self.wheat_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(self.sheep_logo.relative_rect.right + SPACING, SPACING, BANK_CARD_WIDTH,
                                      BANK_HALF_HEIGHT),
            image_surface=self.card_images["Wheat"],
            manager=self.manager,
            container=self.bank_panel)

        self.ore_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(self.wheat_logo.relative_rect.right + SPACING, SPACING, BANK_CARD_WIDTH,
                                      BANK_HALF_HEIGHT),
            image_surface=self.card_images["Ore"],
            manager=self.manager,
            container=self.bank_panel)

        self.dev_logo = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(self.ore_logo.relative_rect.right + SPACING, SPACING, BANK_CARD_WIDTH,
                                      BANK_HALF_HEIGHT),
            html_text="D",
            manager=self.manager,
            container=self.bank_panel)

        self.update_bank_resources()

        self.help_button = pg_g.elements.UIButton(
            text="Help",
            manager=self.manager,
            relative_rect=pygame.Rect(self.card_panel.relative_rect.left + SPACING, self.card_panel.relative_rect.top - SPACING - 50, 50, 50),
            anchors={'left': 'left',
                     'right': 'left',
                     'top': 'bottom',
                     'bottom': 'bottom'}
        )

        self.end_turn_button.disable()
        self.development_button.disable()
        self.city_button.disable()
        self.settlement_button.disable()
        self.road_button.disable()
        self.trade_button.disable()
        self.card_hand = []
        self.set_cards(self.card_hand)

    def update_bank_resources(self):
        if self.wood_bank is not None:
            self.wood_bank.kill()
        self.wood_bank = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(self.wood_logo.relative_rect.left, self.wood_logo.relative_rect.bottom + SPACING,
                                      BANK_CARD_WIDTH,
                                      BANK_HALF_HEIGHT),
            html_text=str(self.bank.lumber),
            manager=self.manager,
            container=self.bank_panel)
        if self.brick_bank is not None:
            self.brick_bank.kill()
        self.brick_bank = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(self.brick_logo.relative_rect.left,
                                      self.brick_logo.relative_rect.bottom + SPACING, BANK_CARD_WIDTH,
                                      BANK_HALF_HEIGHT),
            html_text=str(self.bank.brick),
            manager=self.manager,
            container=self.bank_panel)

        if self.sheep_bank is not None:
            self.sheep_bank.kill()
        self.sheep_bank = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(self.sheep_logo.relative_rect.left,
                                      self.sheep_logo.relative_rect.bottom + SPACING, BANK_CARD_WIDTH,
                                      BANK_HALF_HEIGHT),
            html_text=str(self.bank.wool),
            manager=self.manager,
            container=self.bank_panel)

        if self.wheat_bank is not None:
            self.wheat_bank.kill()
        self.wheat_bank = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(self.wheat_logo.relative_rect.left,
                                      self.wheat_logo.relative_rect.bottom + SPACING, BANK_CARD_WIDTH,
                                      BANK_HALF_HEIGHT),
            html_text=str(self.bank.grain),
            manager=self.manager,
            container=self.bank_panel)

        if self.ore_bank is not None:
            self.ore_bank.kill()
        self.ore_bank = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(self.ore_logo.relative_rect.left, self.ore_logo.relative_rect.bottom + SPACING,
                                      BANK_CARD_WIDTH, BANK_HALF_HEIGHT),
            html_text=str(self.bank.ore),
            manager=self.manager,
            container=self.bank_panel)

        if self.dev_bank is not None:
            self.dev_bank.kill()
        self.dev_bank = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(self.dev_logo.relative_rect.left, self.dev_logo.relative_rect.bottom + SPACING,
                                      BANK_CARD_WIDTH, BANK_HALF_HEIGHT),
            html_text=str(self.bank.totalDevelopment),
            manager=self.manager,
            container=self.bank_panel)

    def handle_board_actions(self):
        if self.board_updated and self.board_action == "initialsettlement":
            self.set_butts = self.initial_settlement_buttons()
            self.board_updated = False
        if self.board_updated and self.board_action == "road":
            self.road_butts = self.get_road_buttons()
            self.board_updated = False
        if self.board_updated and self.board_action == "roadfree":
            self.road_butts = self.get_road_buttons(True)
            self.board_updated = False
        if self.board_updated and self.board_action == "robber":
            self.robber_butts = self.get_robber_buttons()  # todo why is this uninitialized
            self.board_updated = False
        if self.board_updated and self.board_action == "set":
            self.set_butts = self.get_settlement_buttons()
            self.board_updated = False
        if self.board_updated and self.board_action == "city":
            self.set_butts = self.get_city_buttons()
            self.board_updated = False
        if self.board_updated and self.board_action == "initialroad":
            self.road_butts = self.initial_road_buttons(self.property_list[-1])
            self.board_updated = False
        if self.board_action != "turn":
            self.end_turn_button.disable()
            self.development_button.disable()
            self.city_button.disable()
            self.settlement_button.disable()
            self.road_button.disable()
            self.trade_button.disable()
        elif self.board_action == "turn" and self.board_updated:
            self.end_turn_button.enable()
            if Actions.buyDevelopmentCheck(self.player):
                self.development_button.enable()
            else:
                self.development_button.disable()
            if len(Actions.buildCityAvailable(self.player)) != 0:
                self.city_button.enable()
            else:
                self.city_button.disable()
            if len(Actions.buildSettlementAvailable(self.player)) != 0:
                self.settlement_button.enable()
            else:
                self.settlement_button.disable()
            if Actions.buildRoadCheck(self.player):
                self.road_button.enable()
            else:
                self.road_button.disable()
            self.trade_button.enable()
            self.board_updated = False

    def handle_mouse_motion(self, event):
        if self.board_action == "initialsettlement" or self.board_action == "set" or self.board_action == "city":
            for butt in self.set_butts:
                butt.on_mousemotion(event)
        elif self.board_action == "initialroad" or self.board_action == "road" or self.board_action == "roadfree":
            for butt in self.road_butts:
                butt.on_mousemotion(event)
        elif self.board_action == "robber":
            for butt in self.robber_butts:
                butt.on_mousemotion(event)

    def handle_mouse_button_down(self, event):
        if self.board_action == "initialsettlement" or self.board_action == "set" or self.board_action == "city":
            for butt in self.set_butts:
                butt.on_mousebuttondown(event)
        elif self.board_action == "initialroad" or self.board_action == "road" or self.board_action == "roadfree":
            for butt in self.road_butts:
                butt.on_mousebuttondown(event)
        elif self.board_action == "robber":
            for butt in self.robber_butts:
                butt.on_mousebuttondown(event)

    def handle_user_event(self, event):
        if event.user_type == pg_g.UI_BUTTON_PRESSED:
            if self.card_picker is not None:
                self.card_picker.handle_ui_button_pressed(event)
            if self.card_trader is not None:
                self.card_trader.handle_ui_button_pressed(event)
            if event.ui_element == self.help_button:
                if self.help_window is None:
                    self.help_window = help_window((10, 10), self.manager)
                else:
                    if not self.help_window.alive():
                        self.help_window.kill()
                        self.help_window = None
                        self.help_window = help_window((10, 10), self.manager)
                    else:
                        self.help_window.kill()
                        self.help_window = None
            elif event.ui_element == self.trade_button:
                if self.card_trader is None:
                    def callback(trade_with, give_cards, take_cards):
                        # todo check trade_with to implement player trading
                        self.trade_cards(give_cards, take_cards)
                        self.card_trader.kill()
                        self.card_trader = None

                    self.card_trader = TradeGui((50, 100), self.player.resourceHand, self.player.bankTrading, self.manager, callback)
                else:
                    self.card_trader.kill()
                    self.card_trader = None
            elif event.ui_element == self.end_turn_button:
                self.board_input.put("end")
            elif event.ui_element == self.road_button:
                self.set_board_action("road", 'turn')
            elif event.ui_element == self.settlement_button:
                self.set_board_action("set", 'turn')
            elif event.ui_element == self.city_button:
                self.set_board_action("city", 'turn')
            elif event.ui_element == self.development_button:
                self.board_input.put("dev")
            elif event.ui_element == self.dice_button:
                self.is_rolling = not self.is_rolling
            elif '#knight' in event.ui_element.object_ids and not self.player.development_card_played and self.board_action == "turn":
                self.player.development_card_played = True
                self.card_hand.remove('knight')
                self.player.developmentHand.remove_card('knight')
                self.set_cards(self.card_hand)
                self.set_board_action("robber", 'turn')
                self.player.largestArmy += 1
            elif '#roadBuilding' in event.ui_element.object_ids and not self.player.development_card_played and self.board_action == "turn":
                self.player.development_card_played = True
                self.card_hand.remove('roadBuilding')
                self.player.developmentHand.remove_card('roadBuilding')
                self.set_cards(self.card_hand)
                self.board_input.put('roadroad')
            elif '#yearOfPlenty' in event.ui_element.object_ids and not self.player.development_card_played and self.board_action == "turn":
                self.player.development_card_played = True
                if self.card_picker is None:
                    def callback(card_type):
                        self.card_picker.kill()
                        self.card_picker = None
                        self.get_cards([card_type, card_type])
                        self.card_hand.remove('yearOfPlenty')
                        self.player.developmentHand.remove_card('yearOfPlenty')
                        self.set_cards(self.card_hand)

                    self.card_picker = CardPickerGui((50, 200), self.manager, callback)
            elif '#monopoly' in event.ui_element.object_ids and not self.player.development_card_played and self.board_action == "turn":
                self.player.development_card_played = True
                if self.card_picker is None:
                    def callback(card_type):
                        self.card_picker.kill()
                        self.card_picker = None
                        self.card_hand.remove('monopoly')
                        self.player.developmentHand.remove_card('monopoly')
                        self.get_monopoly(card_type)

                    self.card_picker = CardPickerGui((50, 200), self.manager, callback)
            elif '#victoryPoint' in event.ui_element.object_ids and not self.player.development_card_played and self.board_action == "turn":
                pass

    def run(self, board):
        self.game_board = board
        self.game_board.translate_to_3d()
        self.building_locations = StructureBoard(3, self.game_board.get_array())
        self.initialize_gui()
        self.set_cards(self.card_hand)
        pygame.display.flip()  # todo consider removing
        self.running = True
        self.robber_butts = []
        self.display_dice([random.randint(1, 6), random.randint(1, 6)])

        while self.running:
            time_delta = self.clock.tick(30) / 1000.0
            if self.board_updated:
                self.update_bank_resources()
                self.set_player_info(self.player, self.enemy_list)
            if self.cards_updated:
                self.update_cards()
                self.cards_updated = False

            self.handle_board_actions()

            for event in pygame.event.get():
                self.manager.process_events(event)
                if event.type == pygame.QUIT:
                    self.running = False
                elif event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
                    self.running = False
                elif event.type == pygame.MOUSEMOTION:
                    self.handle_mouse_motion(event)
                elif event.type == pygame.MOUSEBUTTONDOWN:
                    self.handle_mouse_button_down(event)
                elif event.type == pygame.USEREVENT:
                    self.handle_user_event(event)

            self.manager.update(time_delta)

            self.draw_board()
            if self.is_rolling:
                self.display_dice([random.randint(1, 6), random.randint(1, 6)])
            self.manager.draw_ui(self.game_window)

            pygame.display.flip()

    def draw_board(self):
        # paint background
        self.game_window.fill((61, 120, 180))
        # draw hexes
        for tile in self.resource_hexes:
            self.game_window.blit(self.texture_dictionary.get(tile.resource),
                                  self.translate_hex_to_xy(tile.location, self.board_center, int(self.hex_size / 2)))

        # draw Board
        self.game_window.blit(self.object_textures.get('Board'),
                              (int(self.board_center[0] - self.grid_size[0] / 2 + self.hex_size / 2),
                               int(self.board_center[1] - self.grid_size[1] / 2 + self.hex_size / 2) - 1))
        # draw resource numbers
        for tile in self.resource_hexes:
            if tile.resource == 'Desert':
                pass
            else:
                text_surface = self.font.render(str(tile.number), False, (0, 0, 0))
                coord = self.translate_hex_to_xy(tile.location, self.board_center, int(self.hex_size / 2))
                self.game_window.blit(text_surface, (coord[0] + self.hex_size / 2 - text_surface.get_width() / 2,
                                                     coord[1] + self.hex_size / 2 - text_surface.get_height() / 2))
        text_surface = self.font.render(INSTRUCTION_TEXT[self.board_action], False, (255, 255, 255))
        coord = self.translate_hex_to_xy(tile.location, self.board_center, int(self.hex_size / 2))
        self.game_window.blit(text_surface, (0, 0))
        # draw ports
        for port in self.port_hexes:
            self.game_window.blit(self.port_textures.get(port.resource),
                                  self.translate_hex_to_xy(port.location, self.board_center, int(self.hex_size / 2)))

        # draw settlements
        for properties in self.property_list:
            shifted_points = []
            location = CornerNode.translate_settlement((properties.cord1, properties.cord2), self.board_center,
                                                       int(self.hex_size / 2))
            if properties.city:
                for points in self.city_points:
                    shifted_points.append((location[0] + points[0] - self.city_width / 2,
                                           location[1] + points[1] + self.city_height))

                pygame.gfxdraw.filled_polygon(self.game_window, shifted_points, COLORS[properties.color])
            else:
                for points in self.house_points:
                    shifted_points.append((location[0] + points[0] - self.house_width / 2,
                                           location[1] + points[1] + self.house_height))

                pygame.gfxdraw.filled_polygon(self.game_window, shifted_points, COLORS[properties.color])

        # draw roads
        for properties in self.road_list:
            location = EdgeNode.translate_road((properties.cord1, properties.cord2), self.board_center,
                                               self.hex_size / 2)
            shifted_points = []
            # variation 0, 1, 2 = |, /, \

            if properties.cord2 % 2 == 1:
                variation = 0
            elif (properties.cord2 + properties.cord1) % 4 == 1:
                variation = 1
            else:
                variation = 2

            self.road_length = self.hex_size / 4
            self.road_width = self.road_length / 15

            x_off = self.road_length * math.sin(math.pi / 6)
            y_off = self.road_length * math.cos(math.pi / 6)
            ratio = math.sin(math.pi / 3)

            offset = [
                [-self.road_width * 4, -self.road_width * 4],
                [-12 * PROTOTYPE_SCALE, -14 * PROTOTYPE_SCALE],
                [-12 * PROTOTYPE_SCALE, -14 * PROTOTYPE_SCALE]
            ]

            for points in self.road_points[variation]:
                shifted_points.append((location[0] + points[0] + offset[variation][0],
                                       location[1] + points[1] + offset[variation][1]))

            pygame.gfxdraw.filled_polygon(self.game_window, shifted_points, COLORS[properties.color])

        loc = self.translate_hex_to_xy(self.game_board.get_robber().location, self.board_center, int(self.hex_size / 2))
        pygame.draw.circle(self.game_window, (50, 50, 50),
                           (loc[0] + int(self.hex_size / 3), loc[1] + int(self.hex_size / 3)), int(30 * PROTOTYPE_SCALE))

        if self.board_action == "initialsettlement" or self.board_action == "set" or self.board_action == "city":
            for butt in self.set_butts:
                butt.draw(self.game_window)
        if self.board_action == "initialroad" or self.board_action == "road" or self.board_action == "roadfree":
            for butt in self.road_butts:
                butt.draw(self.game_window)
        if self.board_action == "robber":
            for butt in self.robber_butts:
                butt.draw(self.game_window)



import queue
import socket
import sys
import threading

sys.path.insert(0, '../frontiersman')
from frontiersman.gameboard.Board import *
from frontiersman.client.Gui import *
from frontiersman.client.Player import *

sep = '\n'
enemy_list = []


class client:
    @staticmethod
    def get_valid_player_input(query_string):
        while True:
            try:
                innie = int(input(query_string))
            except ValueError:
                print("invalid number of players")
                continue
            if 0 < innie <= 4:
                return innie
            elif innie > 4:
                print("Maximum number of players is 4")
                continue
            elif innie < 0:
                print("number of players cannot be negative")

    @staticmethod
    def read_from_buffer(sock):
        totalMessage = ""
        while sep not in totalMessage:
            totalMessage += sock.recv(1).decode('utf-8')
        return totalMessage[:-1]

    def board_thread(self, board):
        self.client_instance.run(board)

    def __init__(self, host_ip, port_ip, player_name='John', num_players=2):
        board_input = queue.Queue()
        board = Board()
        self.client_instance = Client(board_input)
        self.player = None
        ClientSocket = socket.socket()
        xy = ''
        # Color = input('Choose a color: ')
        print('Waiting for connection')
        try:
            ClientSocket.connect((host_ip, int(port_ip)))
        except socket.error as e:
            print(str(e))
        ClientSocket.send(str.encode(player_name + '\n'))
        Color = ''
        while True:
            Response = self.read_from_buffer(ClientSocket)
            # print(Response)
            if Response == 'host':
                # num_players = self.get_valid_player_input("How many players(including you)? ")  # add error checking from pakuri here
                ClientSocket.send(str.encode(str(num_players) + '\n'))
            elif Response == "quit":
                break
            elif Response == 'set':
                # xy=input("x,y: ")
                self.client_instance.set_board_action("initialsettlement")
                coordinates = board_input.get()
                board_input.task_done()
                settle = self.client_instance.building_locations.claimSettlement(int(coordinates.split(',')[0]),
                                                                                 int(coordinates.split(',')[1]), Color)
                self.client_instance.property_list.append(settle)
                self.client_instance.player.acquireNode(settle)
                port = self.client_instance.game_board.xy_give_port((settle.cord1, settle.cord2))
                #print(settle.cord1, settle.cord2)
                if port is not None:
                    self.client_instance.player.bankTrading.update(port.resource)
                self.client_instance.set_board_updated(True)
                ClientSocket.send(str.encode(coordinates + '\n'))
                ClientSocket.send(str.encode("enemyu," + self.client_instance.player.getSendToEnemies() + '\n'))
            elif Response == "startroad":
                self.client_instance.set_board_action("initialroad")
                coordinates = board_input.get()
                board_input.task_done()
                # player.add
                road = self.client_instance.building_locations.claimRoad(int(coordinates.split(',')[0]),
                                                                         int(coordinates.split(',')[1]), Color)
                self.client_instance.road_list.append(road)
                self.client_instance.player.acquireRoad(road)
                self.client_instance.set_board_updated(True)
                ClientSocket.send(str.encode(coordinates + '\n'))
                ClientSocket.send(str.encode("enemyu," + self.client_instance.player.getSendToEnemies() + '\n'))
            elif Response.split(',')[0] == "set":
                #print("new settlement" + Response.split(',')[1] + Response.split(',')[1])
                self.client_instance.property_list.append(
                    self.client_instance.building_locations.claimSettlement(int(Response.split(',')[1]),
                                                                            int(Response.split(',')[2]),
                                                                            Response.split(',')[3]))
                self.client_instance.set_board_updated(True)
            elif Response.split(',')[0] == "road":
                #print("new road")
                #print("new settlement" + Response.split(',')[1] + Response.split(',')[1])
                self.client_instance.road_list.append(
                    self.client_instance.building_locations.claimRoad(int(Response.split(',')[1]),
                                                                      int(Response.split(',')[2]),
                                                                      Response.split(',')[3]))
                self.client_instance.set_board_updated(True)
            elif Response.split(',')[0] == "city":
                self.client_instance.building_locations.settlements[int(Response.split(',')[1])][
                    int(Response.split(',')[2])].city = True
                self.client_instance.set_board_updated(True)
            elif Response.split(',')[0] == "color":
                #print("hi")
                Color = Response.split(',')[1]
                screen_num=Response.split(',')[2]
                self.client_instance.set_player(Player(player_name, Color, screen_num))
                #print(Color)
            elif Response.split(',')[0] == "bank":
                arr = [int(x) for x in Response.split(',')[1:]]
                self.client_instance.bank.update(arr)
                self.client_instance.set_board_updated(True)
            elif Response.split('|')[0] == "board":
                #print("hello")
                number_list = [int(elem) for elem in Response.split('|')[1].split(',')]
                resource_list = Response.split('|')[2].split(',')
                port_list = Response.split('|')[3].split(',')
                board.generate_land(number_list, resource_list)
                board.generate_ports(port_list)
                self.client_instance.enemy_list = enemy_list
                self.client_instance.board_updated = True
            elif Response.split('|')[0] == "startgui":
                threading.Thread(target=self.board_thread, args=(board,)).start()
                ClientSocket.send(str.encode("started\n"))
            elif Response == "getstart":
                #print("get starting resources")
                self.client_instance.start_resources()
                message = board_input.get()
                board_input.task_done()
                ClientSocket.send(str.encode(message + '\n'))
                ClientSocket.send(str.encode("enemyu," + self.client_instance.player.getSendToEnemies() + '\n'))
                # bank_update = board_input.get()
                # board_input.task_done()
            elif Response.split(',')[0] == "dice":
                roll = [int(x) for x in Response.split(',')[1:]]
                self.client_instance.display_dice(roll)
                self.client_instance.process_dice(sum(roll))
                message = board_input.get()
                board_input.task_done()
                ClientSocket.send(str.encode(message + '\n'))
                ClientSocket.send(str.encode("enemyu," + self.client_instance.player.getSendToEnemies() + '\n'))
            elif Response == "turn" or Response == "robber":
                self.client_instance.player.development_card_played = False
                if (Response == "robber"):
                    self.client_instance.set_board_action("robber", 'turn')
                else:
                    self.client_instance.set_board_action("turn")
                while True:
                    message = board_input.get()
                    if (message.split(',')[0] == "road"):
                        road = self.client_instance.building_locations.claimRoad(int(message.split(',')[1]),
                                                                                 int(message.split(',')[2]), Color)
                        self.client_instance.road_list.append(road)
                        self.client_instance.player.acquireRoad(road)
                        self.client_instance.player.getLongestRoad(max([int(en.longestRoad) for en in self.client_instance.enemy_list], default=0))
                        self.client_instance.set_board_updated(True)
                        self.client_instance.pay_cards([1, 0, 1, 0, 0])
                        ClientSocket.send(str.encode(message + ',' + Color + '\n'))
                        ClientSocket.send(str.encode("enemyu," + self.client_instance.player.getSendToEnemies() + '\n'))
                    if (message.split(',')[0] == "robber"):
                        self.client_instance.game_board.set_robber(
                            (int(message.split(',')[1]), int(message.split(',')[2])))
                        self.client_instance.player.getLargestArmy(max([int(en.largestArmy) for en in self.client_instance.enemy_list], default=0))
                        self.client_instance.set_board_updated(True)
                        ClientSocket.send(str.encode(message + '\n'))
                        ClientSocket.send(str.encode("enemyu," + self.client_instance.player.getSendToEnemies() + '\n'))
                    if (message.split(',')[0] == "set"):
                        settle = self.client_instance.building_locations.claimSettlement(int(message.split(',')[1]),
                                                                                         int(message.split(',')[2]),
                                                                                         Color)
                        self.client_instance.property_list.append(settle)
                        self.client_instance.player.acquireNode(settle)
                        port = self.client_instance.game_board.xy_give_port((settle.cord1, settle.cord2))
                        if port is not None:
                            self.client_instance.player.bankTrading.update(port.resource)
                        self.client_instance.set_board_updated(True)
                        self.client_instance.pay_cards([1, 1, 1, 0, 1])
                        ClientSocket.send(str.encode(message + ',' + Color + '\n'))
                        ClientSocket.send(str.encode("enemyu," + self.client_instance.player.getSendToEnemies() + '\n'))
                    if (message == "dev"):
                        ClientSocket.send(str.encode(message + '\n'))
                        card = self.read_from_buffer(ClientSocket)
                        #print(card)
                        self.client_instance.get_dev_card(card)
                        self.client_instance.player.developmentHand.add_card(card)
                        if (card == 'victoryPoint'):
                            self.client_instance.player.hiddenVictoryPoints += 1
                        self.client_instance.pay_cards([0, 1, 0, 1, 1])
                        ClientSocket.send(str.encode("enemyu," + self.client_instance.player.getSendToEnemies() + '\n'))
                    if (message.split(',')[0] == "monopoly"):
                        ClientSocket.send(str.encode(message + '\n'))
                        ClientSocket.send(str.encode("enemyu," + self.client_instance.player.getSendToEnemies() + '\n'))
                    if (message.split(',')[0] == "city"):
                        settle = self.client_instance.building_locations.settlements[int(message.split(',')[1])][
                            int(message.split(',')[2])].city = True
                        self.client_instance.player.acquireCity()
                        self.client_instance.set_board_updated(True)
                        self.client_instance.pay_cards([0, 2, 0, 3, 0])
                        ClientSocket.send(str.encode(message + ',' + '\n'))
                        ClientSocket.send(str.encode("enemyu," + self.client_instance.player.getSendToEnemies() + '\n'))
                    elif (message == "end"):
                        if (
                                self.client_instance.player.victoryPoints + self.client_instance.player.hiddenVictoryPoints >= 10):
                            ClientSocket.send(str.encode('winner,' + self.client_instance.player.name + '\n'))
                        else:
                            ClientSocket.send(str.encode(message + '\n'))
                            ClientSocket.send(
                                str.encode("enemyu," + self.client_instance.player.getSendToEnemies() + '\n'))
                        break
                    elif (message.split(',')[0] == "bank"):
                        ClientSocket.send(str.encode(message + '\n'))
                        ClientSocket.send(str.encode("enemyu," + self.client_instance.player.getSendToEnemies() + '\n'))

                    elif (message == "roadroad"):
                        board_input.task_done()
                        if (len(Actions.buildRoadAvailable(self.client_instance.player, True)) > 0):
                            self.client_instance.set_board_action("roadfree", 'turn')
                            message = board_input.get()
                            board_input.task_done()
                            road = self.client_instance.building_locations.claimRoad(int(message.split(',')[1]),
                                                                                     int(message.split(',')[2]), Color)
                            self.client_instance.road_list.append(road)
                            self.client_instance.player.acquireRoad(road)
                            self.client_instance.player.getLongestRoad(max([int(en.longestRoad) for en in self.client_instance.enemy_list], default=0))
                            self.client_instance.set_board_updated(True)
                            ClientSocket.send(str.encode(message + ',' + Color + '\n'))
                            ClientSocket.send(
                                str.encode("enemyu," + self.client_instance.player.getSendToEnemies() + '\n'))
                        if (len(Actions.buildRoadAvailable(self.client_instance.player, True)) > 0):
                            self.client_instance.set_board_action("roadfree", 'turn')
                            message = board_input.get()
                            board_input.task_done()
                            road = self.client_instance.building_locations.claimRoad(int(message.split(',')[1]),
                                                                                     int(message.split(',')[2]), Color)
                            self.client_instance.road_list.append(road)
                            self.client_instance.player.acquireRoad(road)
                            self.client_instance.player.getLongestRoad(max([int(en.longestRoad) for en in self.client_instance.enemy_list], default=0))
                            self.client_instance.set_board_updated(True)
                            ClientSocket.send(str.encode(message + ',' + Color + '\n'))
                            ClientSocket.send(
                                str.encode("enemyu," + self.client_instance.player.getSendToEnemies() + '\n'))
                board_input.task_done()

                # while turn:
            elif Response.split(',')[0] == "robber":
                self.client_instance.game_board.set_robber((int(Response.split(',')[1]), int(Response.split(',')[2])))
                self.client_instance.set_board_updated(True)
            elif Response.split(',')[0] == "monopoly":
                self.client_instance.pay_monopoly(Response.split(',')[1])
                self.client_instance.set_board_updated(True)
            elif Response == "notturn":
                self.client_instance.set_board_action("display", 'display')
            elif Response.split(',')[0] == "enemy":
                enemy_list.append(EnemyPlayer(Response.split(',')[1], Response.split(',')[2]))
            elif Response.split(',')[0] == "enemyu":
                arr = Response.split(',')[1:]
                for en in self.client_instance.enemy_list:
                    if en.color == arr[1]:
                        en.updateEnemy(arr)
                        self.client_instance.player.loseLongestRoad(int(en.longestRoad))
                        self.client_instance.player.loseLargestArmy(int(en.longestRoad))
            elif Response.split(',')[0] == "winner":
                self.client_instance.running = False
                break
            elif Response != "":
                #print(Response.split(',')[0])
                pass

        # while True:
        # Input = input('Say Something: ')
        # ClientSocket.send(str.encode(Input))
        # Response = ClientSocket.recv(1024)
        # print(Response.decode('utf-8'))
        # roadMap.printAllSettlements()
        # roadMap.printAllRoads()
        self.client_instance.quit()
        ClientSocket.close()
def main():
    address='127.0.0.1'
    port=1233
    name='Zander'
    num_players=2
    #print(sys.argv)
    if(len(sys.argv)==5):
        address=sys.argv[1]
        port=int(sys.argv[2])
        name=sys.argv[3]
        num_players=int(sys.argv[4])
    client(address, int(port), name, int(num_players))


if __name__ == "__main__":
#     host = '127.0.0.1'
#     port = 1233
#     xy = ''
#     Name = "Name"
#     players = 2
    test = client('127.0.0.1', 1233, 'zander', 3)
#     host = sys.argv[1]
#     try:
#         port = int(sys.argv[2])
#     except(ValueError):
#         print("port must be an int")
#     Name = sys.argv[3]
#     if (len(sys.argv) == 5):
#         try:
#             players = int(sys.argv[4])
#         except(ValueError):
#             print("number of players must be an int")
#         test = client(host, port, Name, players)
#     else:
#         test = client(host, port, Name)




